<?php

namespace App\Helpers;

class JaroWinkler
{  private $scalingFactor; // Scaling factor for prefix similarity adjustment

    public function __construct($scalingFactor = 0.1)
    {
        $this->scalingFactor = $scalingFactor;
    }

    /**
     * Compute Jaro-Winkler similarity between two strings.
     *
     * @param string $string1
     * @param string $string2
     * @return float
     */
    public function getSimilarity(string $string1, string $string2): float
    {
        $jaro = $this->calculateJaro($string1, $string2);

        $prefixLength = $this->getCommonPrefixLength($string1, $string2);
        $jaroWinkler = $jaro + ($prefixLength * $this->scalingFactor * (1 - $jaro));

        return round($jaroWinkler, 4);
    }

    /**
     * Calculate Jaro similarity.
     *
     * @param string $string1
     * @param string $string2
     * @return float
     */
    private function calculateJaro(string $string1, string $string2): float
    {
        $len1 = mb_strlen($string1);
        $len2 = mb_strlen($string2);

        if ($len1 === 0 || $len2 === 0) {
            return 0.0;
        }

        $matchDistance = floor(max($len1, $len2) / 2) - 1;

        $matches1 = $this->getMatchingCharacters($string1, $string2, $matchDistance);
        $matches2 = $this->getMatchingCharacters($string2, $string1, $matchDistance);

        $matches = mb_strlen($matches1);

        if ($matches === 0) {
            return 0.0;
        }

        $transpositions = $this->calculateTranspositions($matches1, $matches2);
        $jaro = (1 / 3) * (($matches / $len1) + ($matches / $len2) + (($matches - $transpositions) / $matches));

        return $jaro;
    }

    /**
     * Get matching characters between two strings.
     *
     * @param string $string1
     * @param string $string2
     * @param int $matchDistance
     * @return string
     */
    private function getMatchingCharacters(string $string1, string $string2, int $matchDistance): string
    {
        $len2 = mb_strlen($string2);
        $matched = '';
        $visited = array_fill(0, $len2, false);

        for ($i = 0; $i < mb_strlen($string1); $i++) {
            $start = max(0, $i - $matchDistance);
            $end = min($i + $matchDistance + 1, $len2);

            for ($j = $start; $j < $end; $j++) {
                if (!$visited[$j] && mb_substr($string1, $i, 1) === mb_substr($string2, $j, 1)) {
                    $matched .= mb_substr($string1, $i, 1);
                    $visited[$j] = true;
                    break;
                }
            }
        }

        return $matched;
    }

    /**
     * Calculate transpositions between two matched strings.
     *
     * @param string $matches1
     * @param string $matches2
     * @return int
     */
    private function calculateTranspositions(string $matches1, string $matches2): int
    {
        $transpositions = 0;

        for ($i = 0; $i < mb_strlen($matches1); $i++) {
            if (mb_substr($matches1, $i, 1) !== mb_substr($matches2, $i, 1)) {
                $transpositions++;
            }
        }

        return (int) ($transpositions / 2);
    }

    /**
     * Get the length of the common prefix between two strings.
     *
     * @param string $string1
     * @param string $string2
     * @return int
     */
    private function getCommonPrefixLength(string $string1, string $string2): int
    {
        $maxPrefixLength = 4;
        $prefixLength = 0;

        for ($i = 0; $i < min($maxPrefixLength, mb_strlen($string1), mb_strlen($string2)); $i++) {
            if (mb_substr($string1, $i, 1) === mb_substr($string2, $i, 1)) {
                $prefixLength++;
            } else {
                break;
            }
        }

        return $prefixLength;
    }
}

