<?php

namespace App\Http\Controllers\Auth;

use Carbon\Carbon;
use App\RabbitMQService;
use App\Models\LoginLog;
use App\Jobs\LoginLogJob;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\User;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Redis;
use App\Providers\RouteServiceProvider;
use Illuminate\Support\Facades\Session;
use Illuminate\Validation\ValidationException;
use Illuminate\Foundation\Auth\AuthenticatesUsers;
use PragmaRX\Google2FA\Google2FA;


class LoginController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Login Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles authenticating users for the application and
    | redirecting them to your home screen. The controller uses a trait
    | to conveniently provide its functionality to your applications.
    |
    */

    use AuthenticatesUsers;

    /**
     * Where to redirect users after login.
     *
     * @var string
     */
    protected $redirectTo = RouteServiceProvider::HOME;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest')->except('logout');
    }

    public function username()
    {
        return 'username';
    }

    protected function credentials(Request $request)
    {
        $credentials = $request->only($this->username(), 'password');
        $credentials['active_status'] = STATUS_ON;
        // $credentials['force_change_pass'] = 0; // Add the force_change_pass condition here
        return $credentials;
    }
    protected function sendFailedLoginResponse(Request $request)
    {
        $exception = new \Exception(trans('auth.failed'));
        $this->recordLoginLog($request, 'fail', $exception);
        Session::put('login_error', $exception->getMessage());

        $user = User::where('username', $request->input('username'))
            ->where('active_status', STATUS_OFF)
            ->first();

        if ($user) {
            $message = 'Your account is not activated. Please contact support.';
        } else {
            $message = trans('auth.failed');
        }

        throw ValidationException::withMessages([
            $this->username() => [$message],
        ])->redirectTo($this->redirectTo);
    }

    public function login(Request $request)
    {
        $this->validateLogin($request);
        if ($this->attemptLogin($request)) {
            $this->recordLoginLog($request, 'success');
            return $this->sendLoginResponse($request);
        }
        $this->sendFailedLoginResponse($request);
    }

    protected function recordLoginLog(Request $request, $status, $exception = null)
    {
        // Check if Redis is reachable
        try {
            if (class_exists('Redis') && extension_loaded('redis') && Redis::ping()) {
                $log = [];
                $log['is_logged_in'] = ($status == 'success') ? 1 : 0;
                if ($exception) {
                    $log['error_message'] = $exception->getMessage();
                }
                if (Auth::user()) {
                    $log['profile_id'] = Auth::user()->role->user_profile_id;
                } else {
                    $log['profile_id'] = 0;
                }
                $log['user_id'] =  $request->input('username');
                $log['device_id'] = $request->input('device_id');
                $log['ip_address'] = $request->ip();
                $log['user_agent'] = $request->header('User-Agent');
                $log['login_time '] = Carbon::now();
            }
            if (ENABLE_RABBIT) {
                $log = [];
                $log['is_logged_in'] = ($status == 'success') ? 1 : 0;
                if ($exception) {
                    $log['error_message'] = $exception->getMessage();
                }
                if (Auth::user()) {
                    $log['profile_id'] = Auth::user()->role->user_profile_id;
                } else {
                    $log['profile_id'] = 0;
                }
                $log['user_id'] =  $request->input('username');
                $log['device_id'] = $request->input('device_id');
                $log['ip_address'] = $request->ip();
                $log['user_agent'] = $request->header('User-Agent');
                $log['login_time '] = Carbon::now();

                $rabbit_request_param = ['data' => json_encode($log), 'url' => RABBIT_MQ_LOGIN_LOG_URL, 'name' => 'login_log'];
                $rabbitMQService = new RabbitMQService('login_exchange', 'login_log', 'login_routing_key');
                $rabbitMQService->publish(json_encode($rabbit_request_param));
            }
        } catch (\Exception $e) {
        }
    }
    public function logout(Request $request)
    {
        $this->recordLogoutLog($request, Auth::user());
        Auth::logout();
        $request->session()->invalidate();
        $request->session()->regenerateToken();
        return redirect('/');
    }

    /**
     * Record the logout log.
     *
     * @param  Request  $request
     * @return void
     */
    protected function recordLogoutLog(Request $request, $auth_user)
    {
        $user = $request->user();
        if ($user) {
            // Check if Redis is reachable
            try {
                if (class_exists('Redis') && extension_loaded('redis') && Redis::ping()) {
                    $log = [];
                    $log['is_logged_out'] = 1;
                    $log['user_id'] =  $user->username;
                    $log['device_id'] = $request->input('device_id');
                    $log['profile_id'] = $auth_user->role->user_profile_id;
                    $log['ip_address'] = $request->ip();
                    $log['user_agent'] = $request->header('User-Agent');
                    $log['logout_time '] = Carbon::now();
                }
                if (ENABLE_RABBIT) {
                    $log = [];
                    $log['is_logged_out'] = 1;
                    $log['user_id'] =  $user->username;
                    $log['device_id'] = $request->input('device_id');
                    $log['profile_id'] = $auth_user->role->user_profile_id;
                    $log['ip_address'] = $request->ip();
                    $log['user_agent'] = $request->header('User-Agent');
                    $log['logout_time '] = Carbon::now();
                    $rabbit_request_param = ['data' => json_encode($log), 'url' => RABBIT_MQ_LOGIN_LOG_URL, 'name' => 'login_log'];
                    $rabbitMQService = new RabbitMQService('login_exchange', 'login_log', 'login_routing_key');
                    $rabbitMQService->publish(json_encode($rabbit_request_param));
                }
            } catch (\Exception $e) {
            }
        }
    }
    protected function authenticated(Request $request, $user)
    {
        if ($user->google2fa_secret) {
            return redirect()->route('verify2fa');
        }
        return redirect()->intended($this->redirectPath());
    }
}
