<?php

namespace App\Models;

use DateTime;
use DatePeriod;
use DateInterval;
use Carbon\Carbon;
use App\Traits\RequestResponse;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Config;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Validator;
use Illuminate\Database\Eloquent\Factories\HasFactory;

class AppModel extends Model
{
    use HasFactory, RequestResponse;

    public function generatePagination(Query $query, ServerRequest $request)
    {
        $page = $request->getQuery('start') ? intval($request->getQuery('start')) : 1;
        $limit = ($request->getQuery('length')) ? intval($request->getQuery('length')) : 10;

        $total = $query->count();
        $data = $query->page($page, $limit)->enableHydration(false)->toArray();

        return [
            'recordsTotal' => $total,
            'recordsFiltered' => $total,
            'data' => $data,
        ];
    }

    public function modifiedTempSave($requestdata, $validation = true)
    {
        try {
            if (empty($requestdata['id'])) {
                $response = ['status' => RESPONSE_STATUS_ERROR, 'data' => $requestdata, 'id' => $requestdata['id'], 'message' => "Error"];
            } else {
                $query = $this->findOrFail($requestdata['id']);
                $query['status'] = $requestdata['status'];
                $query['modified'] = Carbon::now();
                if ($query->save()) {
                    $response = ['status' => RESPONSE_STATUS_SUCCESS, 'data' => $requestdata, 'id' => $requestdata['id'], 'message' => "Success"];
                }
            }
        } catch (\Exception $e) {
            return $this->responseWithError('An error occurred.', $e->getMessage());
        }
        return $response;
    }

    public function deleteAll($conditions)
    {
        return $this->where($conditions)->delete();
    }

    public function modifiedSave($requestdata, $validation_rules)
    {
        try {
            if (!array_key_exists('id', $requestdata)) {
                throw new \Exception('Invalid request!', 403);
            }
            if (!empty($validation_rules)) {
                $validator = Validator::make($requestdata, $validation_rules);
                if ($validator->fails()) {
                    return [
                        'status' => RESPONSE_STATUS_ERROR, 'code' => 422,
                        'errors' => $validator->errors(),
                        'message' => __("Validation Error Occured")
                    ];
                }
            }
            $content = $this->updateOrCreate(['id' => $requestdata['id']], $requestdata);
            return ['status' => RESPONSE_STATUS_SUCCESS, 'code' => 200, 'data' => $content, 'id' => $content->id, 'message' => "Saved successfully"];
        } catch (\Exception $e) {
            return ['status' => RESPONSE_STATUS_ERROR, 'code' => 500, 'errorModel' => '', 'message' => $e->getMessage()];
        }
    }

    public function formatRequestData($request_data)
    {
        $columns = $this->getSchema()->typeMap();
        foreach ($columns as $column => $type) {
            if (!isset($request_data[$column])) {
                continue;
            }
            if (in_array($type, ['string', 'text'])) {
                if (is_array($request_data[$column])) {
                    $request_data[$column] = (($request_data[$column]));
                } else {
                    $request_data[$column] = (trim($request_data[$column]));
                }
            } else if (in_array($type, ['integer', 'tinyinteger', 'biginteger'])) {
                $request_data[$column] = intval($request_data[$column]);
            } else if (in_array($type, ['float'])) {
                $float_val = implode("", explode(',', $request_data[$column]));
                $request_data[$column] = floatval($float_val);
            } else if (in_array($type, ['double'])) {
                $double_val = implode("", explode(',', $request_data[$column]));
                $request_data[$column] = doubleval($double_val);
            } else if (in_array($type, ['date']) && $this->checkIsAValidDate($request_data[$column])) {
                $request_data[$column] = date('Y-m-d', strtotime($request_data[$column]));
            } else if (in_array($type, ['datetime']) && $this->checkIsAValidDatetime($request_data[$column])) {
                $request_data[$column] = date('Y-m-d H:i:s', strtotime($request_data[$column]));
            }
        }
        return $request_data;
    }

    function checkIsAValidDate($date)
    {
        $d = \DateTime::createFromFormat('d-m-Y', $date);
        return $d && $d->format('d-m-Y') == $date;
    }

    function checkIsAValidDatetime($datetime)
    {
        $d = \DateTime::createFromFormat('d-m-Y H:i:s', $datetime);
        return $d && $d->format('d-m-Y H:i:s') == $datetime;
    }

    public function updateClientRegStep($newStep, $client_profile_id)
    {
        try {
            $previousStep = $this->where('id', $client_profile_id)->value('meta_data');
            if ($previousStep) {
                foreach ($previousStep as $value) {
                    if ($value['step'] == $newStep['step']) {
                        return ['status' => RESPONSE_STATUS_ERROR, 'status' => 'Already Exist.'];
                    } else {
                        $data = $previousStep;
                        $data[] = $newStep;
                        $finalStep = $data;
                    }
                }
            } else {
                $finalStep = [$newStep];
            }
            return ['status' => RESPONSE_STATUS_SUCCESS, 'data' => $finalStep];
        } catch (\Exception $ex) {
            return ['status' => RESPONSE_STATUS_ERROR, 'message' => $ex->getMessage()];
        }
    }

    public function letterSignatureUpdate($htmlString, $replacement, $pattern, $pattern_current_acceptece_date)
    {
        $replacement_date = date('d-m-Y');
        $htmlString = str_replace($pattern, $replacement, $htmlString);
        return str_replace($pattern_current_acceptece_date, $replacement_date, $htmlString);
    }

    public function elementReplacement($html, $replacement, $pattern)
    {
        return str_replace($pattern, $replacement, $html);
    }

    public function prepareEmailTemplateBody($data, $template_body = '', $template_data = [])
    {
        try {
            $replacements = [];
            if (!empty($template_data)) {
                $keys = ['%TASKDOCUMENTRECEIVEDDATE%', '%TASKENDDATE%', '%TASKSTARTDATE%', '%DOCUMENTLIST%', '%ACCOUNTANCYFIRMNAME%', '%SIGNATORYNAME%', '%PAYROLLQUERYDATES%'];
                foreach ($keys as $key) {
                    $key_as_id = str_replace('%', '_', $key);
                    if (isset($template_data[$key_as_id])) {
                        $replacements[$key] = $template_data[$key_as_id];
                    }
                }
                $data = strtr($template_body, $replacements);
                return $data;
            } else {
                $replacements = [
                    '%TASKNAME%' => $data['task_name'] ?? '(Task Name)',
                    '%SERVICENAME%' => $data['x_service_name'] ?? '(Service Name)',
                    '%ACTIVATIONDATE%' => $data['activation_date'] ?? '(Activation Date)',
                    '%FINISHDATE%' => $data['finish_date'] ?? '(Finish Date)',
                    '%SUBMISSIONDATE%' => $data['submission_date'] ?? '(Submission Date)',
                    '%CLIENTNAME%' => $data['client_name'] ?? '(Client Name)',
                    '%BUSINESSEMAIL%' => $data['business_email'] ?? '(Business Email)',
                    '%BUSINESSPHONE%' => $data['business_phone'] ?? '(Business Phone)',
                    '%CONTACTPERSONNAME%' => $data['contact_person_name'] ?? '(Contact Person Name)',
                    '%CONTACTPERSONMOBILE%' => $data['contact_person_mobile'] ?? '(Contact Person Mobile)',
                    '%CONTACTPERSONEMAIL%' => $data['contact_person_email'] ?? '(Contact Person Email)',
                    '%ACCOUNTANCYFIRMNAME%' => $data['accountancy_firm_name'] ?? '(Accountancy Firm Name)',
                    '%SIGNATORYNAME%' => $data['signatory_name'] ?? '(Signatory Name)',
                    '%SIGNATORYDESIGNATION%' => $data['signatory_designation'] ?? '(Signatory Designation)',
                    '%DOCUMENTLIST%' => $data['document_list'] ?? '(Empty Documents)',
                    '%CURRENTDATE%' => now()->format('d-m-Y') ?? '(Date)',
                    '%TASKSTARTDATE%' => $data['accounting_year_start_date'] ?? '(Task Start Date)',
                    '%TASKENDDATE%' => $data['accounting_year_end_date'] ?? '(Task End Date)',
                    '%TASKDOCUMENTRECEIVEDDATE%' => $data['document_received_date'] ?? '(Document Received Date)',
                ];
                $data = strtr($template_body, $replacements);
                return $data;
            }
        } catch (\Exception $e) {
            return '';
        }
    }

    public function prepareReferenceNo($client_name)
    {
        $exist_match_ref_count = $this->where('reference_number', 'like', $client_name . '%')->count();
        return $client_name . ($exist_match_ref_count + 1);
    }

    public function makeDays($start_date, $end_date)
    {
        $start = new DateTime($start_date);
        $end = new DateTime($end_date);
        $end->modify('+1 day');
        $interval = new DateInterval('P1D'); // 1 day interval
        $daterange = new DatePeriod($start, $interval, $end);
        $dates = [];
        $index = 1;
        foreach ($daterange as $date) {
            $dates[] = array(
                'name' => 'Day ' . $index,
                'start_date' => $date->format('Y-m-d'),
                'end_date' => $date->format('Y-m-d')
            );
            $index++;
        }
        return $dates;
    }

    public function makeWeeks($start_date, $end_date)
    {
        $start_date = new DateTime($start_date);
        $end_date = new DateTime($end_date);
        $weeks = [];
        $index = 1;
        while ($start_date <= $end_date) {
            $week_start = clone $start_date;
            $week_end = clone $week_start;
            $week_end->modify('next sunday');

            if ($week_end > $end_date) {
                $week_end = clone $end_date;
            }

            $weeks[] = array(
                'name' => 'Week ' . $index,
                'start_date' => $week_start->format('Y-m-d'),
                'end_date' => $week_end->format('Y-m-d')
            );

            $start_date = clone $week_end;
            $start_date->modify('+1 day');
            $index++;
        }
        return $weeks;
    }

    public function makeMonths($start_date, $end_date)
    {
        $months = [];

        $start_date = new DateTime($start_date);
        $end_date = new DateTime($end_date);

        while ($start_date <= $end_date) {
            $month_start = clone $start_date;
            $month_end = clone $month_start;
            $month_end->modify('last day of this month');

            if ($month_end > $end_date) {
                $month_end = $end_date;
            }

            $months[] = array(
                'name' => $month_start->format('M'),
                'start_date' => $month_start->format('Y-m-d'),
                'end_date' => $month_end->format('Y-m-d')
            );

            $start_date->modify('first day of next month');
        }
        return $months;
    }

    public function switchDatabase($connection_name, $database, $username, $password, $host = '127.0.0.1', $port = '3306')
    {
        Config::set('database.default', $connection_name);
        Config::set("database.connections.{$connection_name}.database", $database);
        Config::set("database.connections.{$connection_name}.username", $username);
        Config::set("database.connections.{$connection_name}.password", $password);
        Config::set("database.connections.{$connection_name}.host", $host);
        Config::set("database.connections.{$connection_name}.port", $port);

        DB::purge($connection_name);
        DB::reconnect($connection_name);
    }
}
