<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Prunable;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Mail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class EmailService extends AppModel
{
    use HasFactory, Prunable, SoftDeletes;
    protected $connection = 'mysql';
    protected $with = [];
    protected $attributes = ['id' => 0];
    protected $guarded = [];
    protected $dateFormat = 'Y-m-d H:i:s';
    protected $dates = ['mail_sent_time'];
    public static function rules()
    {
        return [
            'email_template_id' => 'required',
            'email_body' => 'required',
        ];
    }

    protected $casts = [
        'mail_sent_time' => 'date:d-M-Y h:i A',
        'created_at' => 'date:d-M-Y',
        'updated_at' => 'date:d-M-Y',
    ];

    public function prunable(): Builder
    {
        return static::where('id', '=', -100000000);
    }

    private function serviceDeletionRoles($query)
    {
        return true;
    }

    public function setMailSentTimeAttribute($value)
    {
        if (empty($value)) {
            $this->attributes['mail_sent_time'] = null;
        } else {
            $this->attributes['mail_sent_time'] = date('Y-m-d H:i:s', strtotime($value));
        }
    }

    private function prepareSearchQuery($query, $request)
    {
        $search_keys = ['id', 'email_template_id', 'receiver_user_id', 'receiver_profile_id', 'sender_user_id', 'sender_profile_id', 'mailer_sent'];
        foreach ($search_keys as $key) {
            if (isset($request[$key])) {
                $query->whereIn($key, is_array(($request[$key])) ? $request[$key] : [$request[$key]]);
            }
        }
        if (isset($request['search_string']) && strlen($request['search_string']) > 0) {
            $query->where('mail_subject', 'like', "%" . $request['search_string'] . "%");
        }
        return $query;
    }

    private function getEmailAttachments($data, $email_service_id)
    {
        $email_attachments = [];
        $m_file_attachment = new FileAttachment();
        $attachment_data = [
            'table_name' => EMAIL_SERVICE_TABLE_NAME,
            'table_row_id' => $data['email_service_id'],
            'x_attachment_type_id' => EMAIL_SEND_ATTACHMENT_TYPE_KEY,
            'verification_status' => 2,
            'verification_date' => now()

        ];
        // Handle the NOTIFICATION_TYPE_COURTESY case separately
        if (isset($data['notify_type']) && $data['notify_type'] == NOTIFICATION_TYPE_COURTESY) {
            $m_file_attachment = new FileAttachment();
            $file_attachment = $m_file_attachment->getAttachment($data['pdf_id']);
            if ($file_attachment) {
                $c_attachment_data = $attachment_data;
                $c_attachment_data['attachment_type'] = $file_attachment['attachment_type'];
                $c_attachment_data['file_name'] = $file_attachment['file_name'];
                $c_attachment_data['original_name'] = $file_attachment['file_name'];
                $c_attachment_data['file_size'] = floatval($file_attachment['file_size']);
                $c_attachment_data['file_dir'] = $file_attachment['file_dir'];
                $c_attachment = $m_file_attachment->saveOrUpdate(setAuthInfoToData($c_attachment_data, Auth::user()));
                if ($c_attachment['status'] === 'success') {
                    $file_path = base64_decode($c_attachment['data']['file_dir']) . $c_attachment['data']['file_name'];
                    $email_attachments[] = ['file' => $file_path];
                }
            }
        }
        if ($email_service_id === null && isset($data['attachment']) && is_array($data['attachment'])) {
            foreach ($data['attachment'] as $attachment) {
                $attachmentResult = $m_file_attachment->imageUploadByForm($attachment_data, $attachment);

                if ($attachmentResult['status'] === 'success') {
                    $file_path = public_path(STORAGE_PATH . base64_decode($attachmentResult['data']['file_dir']) . $attachmentResult['data']['file_name']);
                    $email_attachments[] = ['file' => $file_path];
                }
            }
        } elseif ($email_service_id !== null) {
            $attachments = new FileAttachment();
            $conditions = [
                ['table_name', '=', EMAIL_SERVICE_TABLE_NAME],
                ['table_row_id', '=', $email_service_id],
            ];
            $attachments = $attachments->getConditionWiseAttachments($conditions);
            foreach ($attachments as $attachment) {
                $attachment_data = $m_file_attachment->getAttachmentData($data['email_service_id'], $attachment);
                $attachmentResult = $m_file_attachment->saveOrUpdate(setAuthInfoToData($attachment_data, Auth::user()));
                if ($attachmentResult['status'] === 'success') {
                    $filePathDecode = base64_decode($attachmentResult['data']['file_dir']);
                    $substringToCheck = PDF_ROOT_LOCATION;
                    if (str_contains($filePathDecode, $substringToCheck)) {
                        $file_path = $filePathDecode . $attachmentResult['data']['file_name'];;
                    } else {
                        $file_path = public_path(STORAGE_PATH . $filePathDecode . $attachmentResult['data']['file_name']);
                    }
                    $email_attachments[] = ['file' => $file_path];
                }
            }
        }
        return $email_attachments;
    }

    private function updateEmailStatus($email_service_id, $send_email, $email_response)
    {
        $this->saveOrUpdate(setAuthInfoToData([
            'id' => $email_service_id ?? $send_email['id'],
            'mail_sent_time' => Carbon::now(),
            'mail_sent_status' => 1,
            'mailer_sent' => $email_response['mailer_sent'],
        ], Auth::user()));
    }

    private function getEmailServiceData($email_service)
    {
        return [
            //            "id" => $email_service->id,
            "id" => 0,
            "email_template_id" => $email_service->email_template_id,
            "email_type" => $email_service->email_type,
            "receiver_user_id" => $email_service->receiver_user_id,
            "receiver_profile_id" => $email_service->receiver_profile_id,
            "receiver_name" => $email_service->receiver_name,
            "receiver_email" => $email_service->receiver_email,
            "sender_user_id" => $email_service->sender_user_id,
            "sender_profile_id" => $email_service->sender_profile_id,
            "mail_sent_time" => now(),
            "sender_name" => $email_service->sender_name,
            "sender_email" => $email_service->sender_email,
            "mail_subject" => $email_service->mail_subject,
            "mail_body" => $email_service->mail_body,
        ];
    }

    public function template()
    {
        return $this->hasOne(EmailTemplate::class, 'id', 'email_template_id');
    }

    public function attachments()
    {
        return $this->hasMany(FileAttachment::class, 'table_row_id')->where('table_name', EMAIL_SERVICE_TABLE_NAME);
    }

    public function send_profile()
    {
        return $this->hasOne(UserProfile::class, 'id', 'sender_profile_id');
    }

    public function getEmailService($id)
    {
        return $this->findOrFail($id);
    }

    public function getList($page, $limit, $start, $search)
    {
        $search['search_string'] = $search['value'] ?? '';
        if ($search['type'] == 'inbox') {
            $search['receiver_profile_id'] = $search['profile_id'];
        } else {
            $search['sender_profile_id'] = $search['profile_id'];
        }
        $query = $this::with('template', 'attachments');
        $query = $this->prepareSearchQuery($query, $search);
        $_total = $query->count();
        $emails = $query->limit($limit)->offset($start)->get()->toArray();
        $final_data = [];
        $i = 0;
        foreach ($emails as $email) {
            $index = (++$i) + $start;
            $row = [
                'sl' => $index,
                'id' => $email['id'],
                'name' => (isset($search['type']) && $search['type'] == 'outbox') ? $email['receiver_name'] : $email['sender_name'],
                'email' => (isset($search['type']) && $search['type'] == 'outbox') ? $email['receiver_email'] : $email['sender_email'],
                'mail_subject' => $email['mail_subject'],
                'attachment_count' =>  count($email['attachments']),
                'mail_sent_time' =>  $email['mail_sent_time'],
                'template' =>  !empty($email['template']) ? $email['template']['email_title'] : 'N/A',
                'action' => '',
            ];
            $final_data[] = $row;
        }
        return ['draw' => $page, 'recordsTotal' => $_total, 'recordsFiltered' => $_total, 'data' => $final_data];
    }

    public function sendEmail($email_subject, $email_body, $email_name, $email_to, $attachments = [], $user_id = 0)
    {
        if (MAILER_ENABLE == true) {
            $email = $this->sendEmailMailer($email_subject, $email_body, $email_name, $email_to, $attachments, $user_id);
            $email['mailer_sent'] = 1;
        } else {
            $email = $this->sendEmailDefault($email_subject, $email_body, $email_name, $email_to, $attachments, $user_id);
            $email['mailer_sent'] = 2;
        }
        return $email;
    }

    public function sendEmailDefault($email_subject, $email_body, $email_name, $email_to, $attachments = [])
    {
        $email_to = '';
        $emailData = [
            'email_subject' => $email_subject,
            'email_body' => $email_body,
            'receiver_info' => [
                [
                    "officer" => $email_name,
                    "office_unit" => "",
                    "designation" => "",
                    "officer_email" => $email_to
                ]
            ],
            'attachments' => $attachments
        ];

        if (Mail::to($email_to)->send(new EmailMessage($emailData))) {
            return [
                'status' => 'success',
                'message' => 'Send email successfully'
            ];
        } else {
            return [
                'status' => 'error',
                'message' => 'Email sending failed'
            ];
        }
    }

    public function sendEmailMailer($email_subject, $email_body, $email_name, $email_to, $attachments = [], $user_id = 0)
    {
        try {
            $email_config_data = [];
            $email_setup_data = [];
            if (!empty($email_config_data)) {
                $email_setup_data['protocol'] = $email_config_data['email_protocol'];
                $email_setup_data['username'] = $email_config_data['domain_username'];
                $email_setup_data['password'] = $email_config_data['domain_password'];
                $email_setup_data['host'] = $email_config_data['domain_host'];
                $email_setup_data['port'] = $email_config_data['domain_port'];
                $email_setup_data['secure'] = 'tls';
            }
            $defined_email_to = (in_array(APP_MODE, [APP_MODE_TEST, APP_MODE_DEMO, APP_MODE_PRODUCTION])) ? RECEIVERS_EMAIL : $email_to;
            if (in_array($email_to, $defined_email_to)) {
                $email_to = $email_to;
            } else {
                $email_to = RECEIVER_EMAIL;
            }
            $data['email_subject'] = $email_subject;
            $data['email_body'] = $email_body;
            $data['email_setup_data'] = $email_setup_data;
            $data['receiver_info'] = [0 => ["officer" => $email_name, "office_unit" => "", "designation" => "", "officer_email" => $email_to]];
            $data['attachments'] = $attachments;
            $s_data['data'] = json_encode([
                'service_id' => 10,
                'layout' => 0,
                'template' => 0,
                'email_trace_id' => 0,
                'email_type' => 0,
                'email_type_label' => "",
                'email_from' => FROM_EMAIL,
                'email_from_name' => EMAIL_FROM_NAME,
                'name' => "",
                'response_url' => "",
                'action_type' => 'bse_send',
                'onulipi_info' => []
            ] + $data);
            $response = Http::withoutVerifying()->post(config('paths.email_queue_url'), $s_data);
            if ($response->successful()) {
                return ['status' => RESPONSE_STATUS_SUCCESS, 'message' => 'Send email successfully'];
            } else {
                return ['status' => RESPONSE_STATUS_ERROR, 'message' => 'Email sending failed'];
            }
        } catch (\Exception $e) {
            return ['status' => RESPONSE_STATUS_ERROR, 'message' => $e->getMessage()];
        }
    }

    public function saveOrUpdate(array $data)
    {
        unset($data['current_role']);
        unset($data['modified_by']);
        unset($data['entity_profile_id']);
        unset($data['current_profile_id']);
        return $this->modifiedSave($data, []);
    }


    public function prepareSendEmailData($data, $email_service_id = null)
    {
        try {
            $m_email_template = new EmailTemplate();
            $m_user = new User();
            if ($email_service_id === null) {
                if (empty($data['to_email'])) {
                    throw new \Exception(__("To email filed required!"));
                }
                if (empty($data['user_id'])) {
                    throw new \Exception(__("Invalid request!"));
                }

                $email_template = $m_email_template->getTemplate($data['email_template_id']);
                $receiver_user = $m_user->userDetails($data['user_id']);
                $sender_user = $m_user->userDetails(auth()->id());

                $emailData = [
                    "email_template_id" => $data['email_template_id'],
                    "email_type" => $email_template->email_type ?? '',
                    "receiver_user_id" => $receiver_user->id,
                    "receiver_profile_id" => $receiver_user->profile->id,
                    "receiver_name" => $receiver_user->profile->full_name ?? '',
                    "receiver_email" => $data['to_email'] ?? $receiver_user->profile->email,
                    "sender_user_id" => $sender_user->id,
                    "sender_profile_id" => $sender_user->profile->id,
                    "sender_name" => $sender_user->profile->full_name ?? '',
                    "sender_email" => $sender_user->profile->email,
                    "mail_subject" => $data['compose_subject'] ?? $email_template->email_title,
                    "mail_body" => $data['email_body'],
                    "mail_sent_time" => now(),
                    "mail_sent_status" => 0,
                ];
            } else {
                $email_service = $this->getEmailService($email_service_id);
                if (!$email_service) {
                    throw new \Exception(__("Invalid request!"));
                }
                $emailData = $this->getEmailServiceData($email_service);
            }

            $send_email = $this->saveOrUpdate(setAuthInfoToData($emailData, Auth::user()));
            if ($send_email['status'] != 'success') {
                return ['status' => RESPONSE_STATUS_ERROR, 'message' => $send_email['error']];
            }
            $data['email_service_id'] = $send_email['id'];
            $email_attachments = $this->getEmailAttachments($data, $email_service_id);
            $email_response = $this->sendEmail(
                $emailData["mail_subject"],
                $emailData['mail_body'],
                $emailData['receiver_name'],
                $emailData['receiver_email'],
                $email_attachments
            );
            if (isset($email_response['status']) && $email_response['status'] === 'success') {
                $this->updateEmailStatus($email_service_id, $send_email, $email_response);
            }
        } catch (\Exception $e) {
            return ['status' => RESPONSE_STATUS_ERROR, 'message' => $e->getMessage()];
        }
        return ['status' => RESPONSE_STATUS_SUCCESS, 'data' => [], 'message' => __("Mail sent successfully")];
    }

    public function sendQueryEmail($request_data)
    {
        try {
            $data["mail_subject"] = $request_data['email_subject'] ?? "";
            $data["mail_body"] = $request_data["default_template_content"] ?? "";
            $data["receiver_name"] = $request_data["client_name"] ?? "";
            $data["receiver_email"] = $request_data["client_to_email"] ?? "";
            $email_attachments = [];
            $send_email = $this->sendEmail($data["mail_subject"], $data["mail_body"], $data["receiver_name"], $data["receiver_email"], $email_attachments);
            if ($send_email["status"] == "success") {
                $message = "Email is sent successfully.";
            }
            return ["status" => "success", "message" => $message];
        } catch (\Exception $ex) {
            return ["status" => "error", "message" => $ex->getMessage()];
        }
    }

    public function deleteEmailService($id)
    {
        $query = $this->getEmailService($id);
        $delete_role = $this->serviceDeletionRoles($query);
        if ($delete_role === true) {
            $query->delete();
            return ['message' => __('Deleted Successfully'), 'status' => 'success'];
        } else {
            return ['message' => __($delete_role), 'status' => 'error'];
        }
    }

    public function bulkMailSent($request_data)
    {
        $email_data = [];
        foreach ($request_data as $item) {
            $email_to = (APP_MODE == 'TEST') ? RECEIVER_EMAIL :  $item['receiver_email'];
            $email_data[] = [
                'email_service_id' => $item['id'],
                'email_subject' => $item['email_subject'],
                'token' => isset($item['token']) ? $item['token'] : '',
                'email_body' => $item['email_body'],
                'receiver_info' => ["officer" => $item['receiver_name'], "office_unit" => "", "designation" => "", "officer_email" => $email_to],
                'attachments' => $item['attachments'],
            ];
        }
        try {
            $s_data['data'] = json_encode([
                'service_id' => 10,
                'layout' => 0,
                'template' => 0,
                'email_trace_id' => 0,
                'email_type' => 0,
                'email_type_label' => "",
                'email_from' => FROM_EMAIL,
                'email_from_name' => config('app.name'),
                'name' => "",
                'response_url' => "",
                'action_type' => 'base_send',
                'onulipi_info' => [],
                'receivers' => $email_data,
            ]);
            // $response = Http::withoutVerifying()->post(config('paths.bulk_email_queue_url'), $s_data);
            // if ($response->successful()) {
            //     $responseData = $response->json();
            //     if ($responseData['status'] == 200) {
            //         return ['status' => "success", 'message' => 'Send email successfully'];
            //     } else {
            //         return ['status' => "error", 'message' => $responseData['message']];
            //     }
            // } else {
            //     return ['status' => "error", 'message' => 'Email sending failed'];
            // }
            return ['status' => 'success', 'message' => 'Successfully sent to mail queue server.'];
        } catch (\Exception $e) {
            return ['status' => 'error', 'message' => $e->getMessage(), 'trace' => $e->getTraceAsString()];
        }
    }

    public function bulkResendEmailService()
    {
        try {
            $query = $this->select("*");
            $email_services = $this->prepareSearchQuery($query, ['mailer_sent' => STATUS_OFF])->get()->toArray();
            $email_data = [];
            foreach ($email_services as $key => $item) {
                $email_service['id'] = $item['id'];
                $email_service['retry_count'] = $item['retry_count'] + 1;
                DB::beginTransaction();
                $response = $this->saveOrUpdate($email_service);
                if ($response['status'] == 'error') {
                    DB::rollBack();
                    Log::error('resend.email.template', [
                        'message' => 'Template Resend: ' . $response['message'],
                        'trace' => 'When template resend.',
                    ]);
                    return ['status' => 'error', 'message' => 'Template Resend: ' . $response['message']];
                }

                $email_data[] = [
                    'id' => $item['id'],
                    'token' => customGenerateToken(['now' => now()->timestamp], $response['id']),
                    'email_subject' => $item['mail_subject'],
                    'email_body' => $item['mail_body'],
                    'receiver_name' => $item['receiver_name'],
                    'receiver_email' => $item['receiver_email'],
                    'attachments' => [],
                ];
            }
            if (!empty($email_data)) {
                $response = $this->bulkMailSent($email_data);
                if ($response['status'] == 'error') {
                    DB::rollBack();
                    Log::error('resend.email.template', [
                        'message' => 'Error in bulk email sending: ' . $response['message'],
                        'trace' => 'When bulk mail sending.',
                    ]);
                    return ['status' => 'error', 'message' => 'Error in bulk email sending: ' . $response['message']];
                }
            } else {
                $response = ['status' => 'success', 'message' => 'No queue found.'];
            }
            DB::commit();
            return $response;
        } catch (\Exception $e) {
            DB::rollBack();
            Log::error('resend.email.template', [
                'message' => $e->getMessage(),
                'trace' => $e->getTraceAsString(),
            ]);
            return ['status' => 'error', 'message' => $e->getMessage()];
        }
    }

    public function emailServiceResponseUpdate(array $data)
    {
        try {
            if (empty($data['token'])) {
                return ['status' => RESPONSE_STATUS_ERROR, 'message' => __('Authentication code is mandatory!')];
            }
            if ($data['token']) {
                $payload = customDecodeToken($data['token']);
                if ($payload['data']['enctypt_data'] != $data['email_service_id']) {
                    return ['status' => RESPONSE_STATUS_ERROR, 'message' => __('Invalid authentication code!')];
                }
            }
            if (empty($data['email_service_id'])) {
                return ['status' => RESPONSE_STATUS_ERROR, 'message' => __('Email service_id is mandatory!')];
            }
            if (empty($data['status'])) {
                return ['status' => RESPONSE_STATUS_ERROR, 'message' => __('Response status is mandatory!')];
            }
            if (!in_array($data['status'], [RESPONSE_STATUS_SUCCESS, RESPONSE_STATUS_ERROR])) {
                return ['status' => RESPONSE_STATUS_ERROR, 'message' => __('Response status must be success/error!')];
            }
            $query = $this->select('id', 'mailer_sent');
            $email_service = $this->prepareSearchQuery($query, ['id' => $data['email_service_id']])->first();
            if (empty($email_service)) {
                return ['status' => RESPONSE_STATUS_ERROR, 'message' => __('Invalid Request! Service not found!')];
            }
            if ($email_service->mailer_sent == STATUS_ON) {
                return ['status' => RESPONSE_STATUS_SUCCESS, 'message' => __('Already Sent!')];
            }
            $email_service_data['id'] = $data['email_service_id'];
            $email_service_data['mailer_sent'] = $data['status'] == RESPONSE_STATUS_SUCCESS ? 1 : 0;
            $email_service_data['response_status'] = isset($data['message']) ? $data['message'] : '';
            DB::beginTransaction();
            $response = $this->saveOrUpdate($email_service_data);
            unset($response['id'], $response['data']);
            DB::commit();
            return response()->json($response);
        } catch (\Exception $e) {
            DB::rollBack();
            Log::error('response.service.email', [
                'message' => $e->getMessage(),
                'trace' => $e->getTraceAsString(),
            ]);
            return response()->json(['status' => RESPONSE_STATUS_ERROR, 'message' => $e->getMessage()], 500);
        }
    }
}
