<?php

namespace App\Models;

use Carbon\Carbon;
use DateTime;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Prunable;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Auth;
use Illuminate\Validation\Rule;
use Illuminate\Database\Eloquent\Factories\HasFactory;

class EmailTemplate extends AppModel
{
    use HasFactory, Prunable, SoftDeletes;
    protected $connection = 'mysql';
    protected $with = [];
    protected $attributes = ['id' => 0];
    protected $guarded = ['type'];
    protected $dateFormat = 'Y-m-d H:i:s';
    protected $dates = [];
    public static function rules($id = null)
    {
        return [
            'email_type' => [
                'required',
                'string',
                'max:255',
                Rule::unique('email_templates', 'email_type')->ignore($id),
            ],
            'email_title' => 'required|string|max:255',
            'email_body' => 'required',
        ];
    }

    protected $casts = [
        'created_at' => 'date:d-m-Y',
        'updated_at' => 'date:d-m-Y',
        'template_elements' => 'array',
    ];

    public function prunable(): Builder
    {
        return static::where('id', '=', -100000000);
    }

    private function prepareSearchQuery($query, $request)
    {
        $search_keys = ['id', 'status', 'email_type'];
        foreach ($search_keys as $key) {
            if (isset($request[$key])) {
                $query->whereIn($key, is_array(($request[$key])) ? $request[$key] : [$request[$key]]);
            }
        }
        if (isset($request['search_string']) && strlen($request['search_string']) > 0) {
            $query->where('email_title', 'like', "%" . $request['search_string'] . "%");
        }
        return $query;
    }

    private function templateDeletionRoles($query)
    {
        return true;
    }

    public function loginCredentialsTemplate($full_name, $username, $password)
    {
        $query = $this->select(['*']);
        $template = $this->prepareSearchQuery($query, ['id' => SHARE_CREDENTIAL_TEMPLATE_ID])->first();
        $share_credential_template = $template->email_body;

        $system_image = asset('img/email_header.png');
        $pattern = '%APPLICATIONLOGO%';
        $logo_replace = $this->elementReplacement($share_credential_template, $system_image, $pattern);
        $pattern = '%USERFULLNAME%';
        $full_name_replace = $this->elementReplacement($logo_replace, $full_name, $pattern);
        $pattern = '%USERNAME%';
        $username_replace = $this->elementReplacement($full_name_replace, $username, $pattern);
        $pattern = '%PASSWORD%';
        $password_replace = $this->elementReplacement($username_replace, $password, $pattern);
        $pattern = '%APPLICATIONNAME%';
        $af_name_replace = $this->elementReplacement($password_replace, APP_NAME, $pattern);
        $pattern = '%LOGINURL%';
        $final_html = $this->elementReplacement($af_name_replace, SYSTEM_URL, $pattern);
        return $final_html;
    }

    public function vatReturnTemplate($request_data, $template_id)
    {
        $query = $this->select(['*']);
        $template = $this->prepareSearchQuery($query, ['id' => $template_id])->first();
        $vat_return_template = $template->email_body;

        $vat_return_quarter_end_date = $request_data['vat_return_quarter_end_date'];
        $pattern = '%VATRETURNQUARTERENDDATE%';
        $vat_return_quarter_replace = $this->elementReplacement($vat_return_template, $vat_return_quarter_end_date, $pattern);
        $amount = $request_data['amount'];
        $pattern = '%AMOUNT%';
        $amount_replace = $this->elementReplacement($vat_return_quarter_replace, $amount, $pattern);
        $payment_ref = $request_data['payment_ref'];
        $pattern = '%PAYMENTREF%';
        $paymenr_ref_replace = $this->elementReplacement($amount_replace, $payment_ref, $pattern);
        $hmrc_sort_code = $request_data['hmrc_sort_code'];
        $pattern = '%HMRCSORTCODE%';
        $hmrc_sort_code_replace = $this->elementReplacement($paymenr_ref_replace, $hmrc_sort_code, $pattern);
        $hmrc_account_number = $request_data['hmrc_account_number'];
        $pattern = '%HMRCACCOUNTNUMBER%';
        $hmrc_account_number_replace = $this->elementReplacement($hmrc_sort_code_replace, $hmrc_account_number, $pattern);
        $hmrc_account_name = $request_data['hmrc_account_name'];
        $pattern = '%HMRCACCOUNTNAME%';
        $hmrc_account_name_replace = $this->elementReplacement($hmrc_account_number_replace, $hmrc_account_name, $pattern);
        $afp_company_name = $request_data['afp_company_name'];
        $pattern = '%ACCOUNTANCYFIRMNAME%';
        $final_html = $this->elementReplacement($hmrc_account_name_replace, $afp_company_name, $pattern);
        return $final_html;
    }

    public function getTemplate($id)
    {
        try {
            return $this->findOrFail($id);
        } catch (\Exception $e) {
            return ['status' => 'error', 'message' => $e->getMessage()];
        }
    }

    public function getEmailTemplateByType($condition, $select = ['*'])
    {
        $query = $this->select($select);
        return $this->prepareSearchQuery($query, $condition);
    }

    public function getTemplates($status = -1)
    {
        $query = $this->select();
        if ($status != -1) {
            $query->where(['status' => intval($status)]);
            $query = $this->prepareSearchQuery($query, ['status' => intval($status)]);
        }
        return $query->orderBy('id', 'desc')->get()->toArray();
    }

    public function deleteTemplate($id)
    {
        $query = $this->getTemplate($id);
        $delete_role = $this->templateDeletionRoles($query);
        if ($delete_role === true) {
            $query->delete();
            return ['message' => __('Deleted Successfully'), 'status' => 'success'];
        } else {
            return ['message' => __($delete_role), 'status' => 'error'];
        }
    }

    public function getList($page, $limit, $start, $search)
    {
        $query = $this->select(['id']);
        $query = $this->prepareSearchQuery($query, ['search_string' => $search['value']]);
        $_total = $query->count();
        $query->limit($limit)->offset($start);
        $emailIds = $query->pluck('id');
        $emails = $this->select([
            'id',
            'email_type',
            'email_title',
            'status'
        ])->whereIn('id', $emailIds)->get()->toArray();
        $final_data = [];
        $i = 0;
        foreach ($emails as $email) {
            $index = (++$i) + $start;
            $row = [
                'sl' => $index,
                'id' => $email['id'],
                'email_type' => $email['email_type'],
                'email_title' => $email['email_title'],
                'email_body' => '',
                'status' => $email['status'],
                'action' => '',
            ];
            $final_data[] = $row;
        }
        return ['draw' => $page, 'recordsTotal' => $_total, 'recordsFiltered' => $_total, 'data' => $final_data];
    }

    public function getOptions($request = null)
    {
        try {
            $query = $this::select('id', 'email_title');
            return $this->prepareSearchQuery($query, ['search_string' => $request['value'] ?? ''])->pluck('email_title', 'id')->toArray();
        } catch (\Exception $e) {
            return [];
        }
    }

    private function prepareRequestData($data)
    {
        $email_data['id'] = $data['id'] ?? 0;
        $email_data['email_type'] = $data['email_type'] ?? null;
        $email_data['email_title'] = $data['email_title'] ?? null;
        $email_data['email_body'] = $data['email_body'] ?? null;
        $email_data['created_by'] = $data['created_by'] ?? null;
        $email_data['updated_by'] = $data['updated_by'] ?? null;
        $email_data['template_elements'] =  !empty($data['template_elements']) ? array_flip(explode(',', $data['template_elements'])) : null;
        return $email_data;
    }

    public function saveOrUpdate(array $data)
    {
        $template_data = $this->prepareRequestData($data);
        return $this->modifiedSave($template_data, []);
    }

    public function saveTemplate($request)
    {
        $form_data = json_decode(base64_decode($request->rd), true);
        $current_user = Auth::user();
        $string = ucwords(strtolower($form_data['email_title']));
        $trimmedString = preg_replace('/\s+/', '', $string);
        $email_type = Str::snake($trimmedString);
        try {
            $request_data['email_type'] = $email_type;
            $request_data['id'] = isset($form_data['id']) ? $form_data['id'] : 0;
            $request_data['email_title'] = $form_data['email_title'];
            $request_data['email_body'] = $form_data['email_body'];
            $request_data['status'] = 'active';
            $request_data['created_by'] = $current_user->id;
            $request_data['modified_by'] = $current_user->id;
            $request_data['created'] = Carbon::now();
            $request_data['modified'] = Carbon::now();
            $response = $this->saveOrUpdate($request_data);
            if ($response['status'] == 'success') {
                return $this->responseSuccess('Template saved successfully.');
            }
        } catch (\Exception $e) {
            // Handle the exception
            return $this->responseWithError('An error occurred while saving Template Element.', $e->getMessage());
        }
    }

    public function shareCredentialData($user_profile_id)
    {
        $m_user_profile = new UserProfile();
        $response_user = $m_user_profile->getUserProfile($user_profile_id);
        $mailPrepareData['APPLICATIONLOGO'] = asset('img/email_header.png');
        $mailPrepareData['SYSTEMLOGO'] = asset('img/email_header.png');
        $mailPrepareData['USERFULLNAME'] = $response_user->full_name ?? '';
        $mailPrepareData['APPLICATIONNAME'] = APP_NAME;
        $mailPrepareData['USERNAME'] = $response_user->user->username ?? '';
        $mailPrepareData['PASSWORD'] = '********';
        $mailPrepareData['LOGINURL'] = SYSTEM_URL;
        return $mailPrepareData;
    }

    public function prepareTemplateData($request_data)
    {
        if (isset($request_data['user_profile_id']) &&  $request_data['user_profile_id'] == 0) {
            $request_data['user_profile_id'] = $request_data['current_profile_id'];
        }
        if (isset($request_data['email_template_id']) && $request_data['email_template_id'] == SHARE_CREDENTIAL_TEMPLATE_ID) {
            $mailPrepareData = $this->shareCredentialData($request_data['user_profile_id']);
            return ['status' => RESPONSE_STATUS_SUCCESS, 'message' => RESPONSE_STATUS_SUCCESS, 'data' => $mailPrepareData];
        } elseif (isset($request_data['email_template_id']) && $request_data['email_template_id'] == DEFAULT_TEMPLATE_LETTER) {
            $m_user_profile = new UserProfile();
            $profile = $m_user_profile->getProfile($request_data['user_profile_id']);
            $mailPrepareData['EMAILRECEIPIENTNAME'] = $profile->full_name ?? '-';
            return ['status' => RESPONSE_STATUS_SUCCESS, 'message' => RESPONSE_STATUS_SUCCESS, 'data' => $mailPrepareData];
        }
         else {
            return ['status' => RESPONSE_STATUS_SUCCESS, 'message' => RESPONSE_STATUS_SUCCESS, 'data' => []];
        }
    }

    public function statusUpdate($request_data)
    {
        return $this->modifiedSave($request_data, []);
    }

    public function defaultEmailTemplate($client_profile_id, $followed_template_id)
    {
        $clientProfile = new ClientProfile();
        $client = $clientProfile->clientListByProfileIds([$client_profile_id], ['id', 'client_name', 'incorporation_number'])->first();
        $template = $this->find($followed_template_id)->email_body;
        $cl_afp = new AccountingFirmProfile();
        $firmProfile = $cl_afp->getFirmProfile();
        $replacements = [
            '%EMAILRECEIPIENTNAME%' => $client->client_name ?? '(Receipient Name)',
            '%SIGNATORYNAME%' => $client->client_responsible->manager->full_name ?? '(Signatory Name)',
            '%SIGNATORYDESIGNATION%' => $client->client_responsible->manager->designation ?? '(Signatory Designation)',
            '%ACCOUNTANCYFIRMNAME%' => $firmProfile['company_name'] ?? '(Accountancy Firm Name)',
        ];
        return strtr($template, $replacements);
    }
}
