<?php

namespace App\Models;

use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\Validator;
use Illuminate\Database\Eloquent\Factories\HasFactory;

class EntityTypeRole extends AppModel
{
    use HasFactory;
    protected $connection = 'mysql';
    protected $with = [];
    protected $dateFormat = 'Y-m-d H:i:s';
    protected $dates = ['finish_date', 'submission_date'];
    public $timestamps = true;
    /**
     * The model's default values for attributes.
     * @var array
     */
    protected $attributes = ['id' => 0];
    /**
     * The attributes that aren't mass assignable.
     * @var array
     */
    protected $guarded = ['type'];
    protected $casts = [
        'entity_type_role_ids' => 'array', // convert array to json and json to array while save and retrive
    ];
    public static function rules()
    {
        return [
            'entity_type_id' => 'required|int',
            'role_name' => 'required|string',
            'role_title' => 'required|string',
            'level' => 'required|int',
        ];
    }

    public function entityType()
    {
        return $this->belongsTo(EntityType::class, 'entity_type_id');
    }

    public function uniqueValidationCheck(array $data)
    {
        $validator = Validator::make($data, [
            'role_name' => [
                'required',
                Rule::unique('entity_type_roles')->ignore($data['id'])->where(function ($query) use ($data) {
                    return $query->where('entity_type_id', $data['entity_type_id'])
                        ->where('role_name', $data['role_name']);
                }),
            ],
        ]);
        if ($validator->fails()) {
            return [
                'status' => 'error', 'code' => 422,
                'errors' => $validator->errors()->toArray(),
                'message' => __("Validation Error Occurred")
            ];
        }
    }

    public function getEntityByRole($id)
    {
        return $this->where('entity_type_id', $id)->pluck('role_title', 'id')->toArray();
    }

    public function getEntityByRoleAndAdmin($id)
    {
        return $this->where('entity_type_id', $id)->orWhere('role_name', strtoupper(ROLE_ADMIN))->pluck('role_title', 'id')->toArray();
    }

    public function sendNotificationUsers($dataArray = [])
    {
        $newDataArray = [];
        if (count($dataArray) > 0) {
            foreach ($dataArray as $id => $message) {
                $notificationEvent = $this->getEntityById($id);
                if ($notificationEvent) {
                    $newDataArray[$notificationEvent->role_name] = $message;
                }
            }
        }
        return $newDataArray;
    }

    public function getEntityById($id)
    {
        return $this->findOrFail($id);
    }

    public function getRoleByNames($ids)
    {
        if (!empty($ids)) {
            $ids = $ids;
            $roles = $this->whereIn('id', $ids)->pluck('role_title')->toArray();
            return implode(', ', $roles);
        }
    }

    public function getEntityTypeRoleList($page, $limit, $start, $search)
    {
        $search = $search['value'];
        $roles = $this::with('entityType')
            ->when($search, function ($query) use ($search) {
                return $query->where('role_name', 'like', "%" . $search . "%")->orWhere('role_title', 'like', "%" . $search . "%");
            })
            ->limit($limit)
            ->offset($start)
            ->get()
            ->toArray();
        $_total = $this::count();
        $final_data = [];
        $i = 0;
        foreach ($roles as $role) {
            $index = (++$i) + $start;
            $row = [
                'sl' => $index,
                'id' => $role['id'],
                'role_name' => $role['role_name'],
                'role_title' => $role['role_title'],
                'level' => $role['level'],
                'entity_type' => $role['entity_type']['name'] ?? 'Super Admin',
                'action' => '',
            ];
            $final_data[] = $row;
        }
        return ['draw' => $page, 'recordsTotal' => $_total, 'recordsFiltered' => $_total, 'data' => $final_data];
    }

    public function saveOrUpdate(array $data)
    {
        $check = $this->uniqueValidationCheck($data);
        if ($check) {
            return $check;
        }
        return $this->modifiedSave($data, $this->validation_rules);
    }

    public function deleteEntityTypeRole($id)
    {
        $entity_type_role = $this::findOrFail($id);
        $response = ['message' => __('Invalid Request'), 'status' => 'error'];
        if ($entity_type_role->delete()) {
            $response = ['message' => __('Deleted Successfully'), 'status' => 'success'];
        }
        return $response;
    }

    public function getEntityTypeByEntityRoleId($entity_type_role_id)
    {
        return $this->where("entity_type_id", $entity_type_role_id)
            ->where("role_name", CLIENT_ROLE_NAME_ADMIN)
            ->first();
    }
}
