<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Prunable;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;

class FileAttachment extends AppModel
{
    use HasFactory, Prunable, SoftDeletes;
    protected $connection = 'mysql';
    protected $with = [];
    protected $attributes = ['id' => 0];
    protected $guarded = ['type'];
    protected $dateFormat = 'Y-m-d H:i:s';
    protected $dates = [];
    protected $appends = ['file_url'];
    public static function rules()
    {
        return [
            'table_name' => 'required|string|max:100',
            'table_row_id' => 'required',
            'attachment_type' => 'required',
            'file_name' => 'required|string|max:255',
        ];
    }

    protected $casts = [
        'created_at' => 'date:d-m-Y  h:i A',
        'updated_at' => 'date:d-m-Y h:i A',
        'verification_date' => 'date:d-m-Y h:i A',
    ];

    public function prunable(): Builder
    {
        return static::where('id', '=', -100000000);
    }

    private function prepareSearchQuery($query, $request)
    {
        $search_keys = ['id', 'table_row_id', 'x_attachment_type_id', 'verification_status'];
        foreach ($search_keys as $key) {
            if (isset($request[$key]) && intval($request[$key]) > 0) {
                $query->whereIn($key, is_array(($request[$key])) ? $request[$key] : [$request[$key]]);
            }
        }
        if (isset($request['search_string']) && strlen($request['search_string']) > 0) {
            $query->where('file_name', 'like', "%" . $request['search_string'] . "%");
        }
        return $query;
    }

    private function attachmentUnlink($filePath)
    {
        if (file_exists($filePath)) {
            unlink($filePath);
        }
    }

    private function attachmentDeletionRoles($query)
    {
        $file_path = storage_path(IMAGE_PREFIX . base64_decode($query->file_dir) . $query->file_name);
        $this->attachmentUnlink($file_path);
        return true;
    }

    private function attachmentsDeletionRoles($queries)
    {
        foreach ($queries->get() as $query) {
            $file_path = storage_path(IMAGE_PREFIX . base64_decode($query->file_dir) . $query->file_name);
            $this->attachmentUnlink($file_path);
        }
        return true;
    }

    public function created_profile()
    {
        return $this->belongsTo(UserProfile::class, 'created_by', 'user_id');
    }
    public function updated_profile()
    {
        return $this->belongsTo(UserProfile::class, 'modified_by', 'user_id');
    }
    public function folders()
    {
        return $this->belongsToMany(XFolder::class, 'file_attachment_x_folder');
    }
    public function x_document()
    {
        return $this->belongsTo(XDocument::class, 'table_row_id');
    }

    public function getFileUrlAttribute()
    {
        $decodedFileDir = base64_decode($this->file_dir);
        if ($this->table_name === CONTRACT_TEMPLATED_DOCUMENT_TABLE_NAME) {
            $newPath = str_replace(PDF_ROOT_LOCATION, '', $decodedFileDir);
            $file_url = config('paths.pdf_url') . $newPath . $this->file_name;
        } else {
            $substringToCheck = PDF_ROOT_LOCATION;
            if (str_contains($decodedFileDir, $substringToCheck)) {
                $newPath = str_replace(PDF_ROOT_LOCATION, '', $decodedFileDir);
                $file_url = config('paths.pdf_url') . $newPath . $this->file_name;
            } else {
                $file_url = asset(STORAGE_PATH . $decodedFileDir . '/' . $this->file_name);
            }
        }
        return $file_url;
    }

    public function getAttachments()
    {
        return $this->latest()->get();
    }

    public function getAttachmentData($table_row_id, $attachment)
    {
        return [
            //            "id" => $attachment->id,
            "id" => 0,
            "table_name" => $attachment->table_name,
            "table_row_id" => $table_row_id,
            "attachment_type" => $attachment->attachment_type,
            "x_attachment_type_id" => $attachment->x_attachment_type_id,
            "file_name" => $attachment->file_name,
            "file_size" => $attachment->file_size,
            "file_dir" => $attachment->file_dir,
            "original_name" => $attachment->original_name,
            "verification_status" => 2,
            "verification_date" => now()
        ];
    }


    public function deleteAttachment($id)
    {
        $query = $this->findOrFail($id);
        $delete_role = $this->attachmentDeletionRoles($query);
        if ($delete_role === true) {
            $file_path = storage_path(IMAGE_PREFIX . base64_decode($query->file_dir) . $query->file_name);
            $this->attachmentUnlink($file_path);
            $query->delete();
            return ['message' => __('Deleted Successfully'), 'status' => 'success'];
        } else {
            return ['message' => __($delete_role), 'status' => 'error'];
        }
    }

    public function deleteAttachmentByTableInfo($table_name, $table_row_id)
    {
        $query = $this->where('table_name', $table_name)->where('table_row_id', $table_row_id);
        $delete_role = $this->attachmentsDeletionRoles($query);
        if ($delete_role === true) {
            $query->delete();
            return ['message' => __('Deleted Successfully'), 'status' => 'success'];
        } else {
            return ['message' => __($delete_role), 'status' => 'error'];
        }
    }

    public function getConditionWiseAttachments($conditions, $orWhereConditions = null)
    {
        if ($orWhereConditions != null) {
            $attachments = $this->where($conditions)->orWhere($orWhereConditions)->get();
        } else {
            $attachments = $this->where($conditions)->get();
        }
        return $attachments;
    }
    public function base64Upload($data)
    {
        $prefix = IMAGE_PREFIX;
        $path_info = IMAGE_PATH . '/' . APP_MODE . '/' . $data['table_name'] . '/' . $data['table_row_id'] . '/';
        $folderPath = storage_path($prefix . $path_info);
        makeDirectory($folderPath, 0775);
        $image_parts = explode(";base64,", $data['image']);
        $image_type_aux = explode("image/", $image_parts[0]);
        $image_type = $image_type_aux[1];
        $image_base64 = base64_decode($image_parts[1]);
        $imageName = uniqid() . '.' . $image_type;
        $imageFullPath = $folderPath . $imageName;
        $file_name = $path_info . $imageName;
        $attachment_old = $this->where('table_name', $data['table_name'])->where('table_row_id', $data['table_row_id'])->where('x_attachment_type_id', $data['x_attachment_type_id'])->first();
        $data['id'] = 0;
        if ($attachment_old) {
            $data['id'] = $attachment_old->id;
            if (!empty($attachment_old->file_dir)) {
                imageUnlink($prefix . base64_decode($attachment_old->file_dir));
            }
            if (!empty($attachment_old->thumb_dir)) {
                imageUnlink($prefix . base64_decode($attachment_old->file_dir));
            }
        }
        if (file_put_contents($imageFullPath, $image_base64)) {
            $data['attachment_type'] = $image_type;
            $data['file_name'] = $imageName;
            $data['file_size'] = (filesize($imageFullPath) / 1024);
            $data['file_dir'] = base64_encode($file_name);
            $data['thumb_dir'] = '';
            $data['original_name'] = !empty($data['original_name']) ? $data['original_name'] : $imageName;
            $response = $this->saveOrUpdate(setAuthInfoToData($data, $this->auth_user));
            if ($response['status'] != "success") {
                imageUnlink($prefix . $file_name);
            }
            return $response;
        } else {
            $response = ['status' => 'error', 'code' => 500, 'message' => __('Failed to save Image')];
        }
        return $response;
    }

    public function imageUploadByForm($data, $file, $path = null, $delete = false)
    {
        try {
            if ($delete) {
                $this->deleteAttachmentByTableInfo($data['table_name'], $data['table_row_id']);
            }
            $prefix = IMAGE_PREFIX;
            $originalName = $file->getClientOriginalName();
            $fileSize = number_format(($file->getSize() / 1024), 2, '.', '');
            $attachmentType = strtolower($file->getClientOriginalExtension());
            $pathInfo = $path ?? IMAGE_PATH . '/' . APP_MODE . '/' .  $data['table_name'] . '/' . $data['table_row_id'] . '/';
            $folderPath = storage_path($prefix . $pathInfo);
            $imageName = uniqid() . '_' . $originalName;

            // Ensure the directory exists or create it
            makeDirectory($folderPath, 0775);

            if ($file->move(storage_path($prefix . $pathInfo), $imageName)) {
                $data['attachment_type'] = $attachmentType;
                $data['file_name'] = $imageName;
                $data['file_size'] = $fileSize;
                $data['file_dir'] = base64_encode($pathInfo);
                $data['thumb_dir'] = '';
                $data['original_name'] = $originalName ?? $imageName;

                if (isset($data['verification_status']) && $data['verification_status'] == 2) {
                    $data['verification_status'] = 2;
                    $data['verification_date'] = now();
                }

                $user = Auth::user();
                $response = $this->saveOrUpdate(setAuthInfoToData($data, $user));

                if ($response['status'] != "success") {
                    $this->attachmentUnlink($prefix . $pathInfo . $imageName);
                }

                return $response;
            } else {
                $response = ['status' => 'error', 'code' => 500, 'message' => __('Failed to save Image'), 'data' => []];
            }

            return $response;
        } catch (\Exception $e) {
            $this->attachmentUnlink($prefix . $pathInfo . $imageName);
            return $this->responseWithError($e->getMessage());
        }
    }

    public function getAttachmentPath($table_name, $attachment_type_id, $table_row_id)
    {
        $attachment = FileAttachment::where('table_row_id', $table_row_id)
            ->where('table_name', $table_name)
            ->where('x_attachment_type_id', $attachment_type_id)
            ->first();
        if ($attachment) {
            $file_name = $attachment->file_name;
            $file_path = base64_decode($attachment->file_dir) . '/' . $file_name;
            return $file_path;
        } else {
            return null;
        }
    }

    public function saveOrUpdate(array $data)
    {
        return $this->modifiedSave($data, []);
    }

    public function nameUpdate($id, $name)
    {
        $attachment = $this->find($id);
        $data['id'] = $id;
        $data['original_name'] = $name . "." . pathinfo($attachment->original_name)['extension'];
        $response = $this->saveOrUpdate(setAuthInfoToData($data, $this->auth_user));
        if ($response['status'] == "success") {
            return true;
        }
        return false;
    }

    public function getAttachment($id)
    {
        return $this->findOrFail($id);
    }

    public function getFileAttachmentByServiceDocIds($client_contract_ids)
    {
        $query = $this->select(['*']);
        return $this->prepareSearchQuery($query, [
            'table_row_id' => $client_contract_ids,
            'table_name' => 'client_service_documents'
        ]);
    }

    public function createAutomationFileAttachemnts($client_service_Document_ids)
    {
        $today = now();
        $old_client_service_Document_ids = array_flip($client_service_Document_ids);
        $file_attachments = $this->getFileAttachmentByServiceDocIds($old_client_service_Document_ids)->get()->toArray();
        $file_attachment_list = [];
        foreach ($file_attachments as $key => $file_attachment) {
            if (in_array($file_attachment['table_row_id'], $old_client_service_Document_ids)) {
                $file_attachment['id'] = 0;
                $file_attachment['table_row_id'] = $client_service_Document_ids[$file_attachment['table_row_id']];
                $file_attachment['verification_status'] = 0;
                $file_attachment['verification_date'] = null;
                $file_attachment['created_at'] = $today;
                $file_attachment['updated_at'] = $today;
            }
            unset($file_attachment['file_url']);
            $file_attachment_list[] = $file_attachment;
        }
        $response = $this->bulkSave($file_attachment_list);
        if ($response['status'] == 'error') {
            return $response['message'];
        }
        return $response;
    }

    public function bulkSave($request_data)
    {
        try {
            DB::beginTransaction();
            $this->insert($request_data);
            DB::commit();
            return ['status' => 'success', 'message' => __('Successfully saved.')];
        } catch (\Exception $e) {
            DB::rollBack();
            return ['status' => 'error', 'message' => $e->getMessage()];
        }
    }

    public function uploadContractDocuments($request_data)
    {
        $attachment_data = [
            'table_name' => CLIENT_SERVICE_DOCUMENT_TABLE_NAME,
            'table_row_id' => $request_data['client_service_document_id'],
            'x_attachment_type_id' => OTHERS_ATTACHMENT_TYPE_KEY,
            'verification_status' => 0,
        ];
        $path = IMAGE_PATH . '/' . APP_MODE . '/' . $attachment_data['table_name'] . '/' . $attachment_data['table_row_id'] . '/';
        $response_file_upload = $this->imageUploadByForm($attachment_data, $request_data['file'], $path, false);
        if ($response_file_upload['status'] == RESPONSE_STATUS_ERROR) {
            return ['status' => RESPONSE_STATUS_ERROR, 'message' => __('Error when save attachment.')];
        }
        return $response_file_upload;
    }

    public function uploadAFDocument($request_data, $x_document_id)
    {
        $attachment_data = [
            'table_name' => X_DOCUMENT_TABLE_NAME,
            'table_row_id' => $x_document_id,
            'x_attachment_type_id' => X_DOCUMENT_ATTACHMENT_TYPE_KEY,
            'verification_status' => 0,
        ];
        $path = IMAGE_PATH . '/' . APP_MODE . '/' . $attachment_data['table_name'] . '/' . $attachment_data['table_row_id'] . '/';
        $response_file_upload = $this->imageUploadByForm($attachment_data, $request_data['file'], $path, false);
        if ($response_file_upload['status'] == RESPONSE_STATUS_ERROR) {
            return ['status' => RESPONSE_STATUS_ERROR, 'message' => __('Error when save attachment.')];
        }
        return $response_file_upload;
    }

    public function getAttachmentsByFilter($conditions, $select = ['*'])
    {
        $query = $this->select($select);
        return $this->prepareSearchQuery($query, $conditions);
    }

    public function uploadVatReturnDocument($request_data, $table_row_id)
    {
        $attachment_data = [
            'table_name' => VAT_RETURN_STATUSES_TABLE_NAME,
            'table_row_id' => $table_row_id,
            'x_attachment_type_id' => VAT_RETURN_ATTACHMENT_TYPE_KEY,
            'verification_status' => 0,
        ];
        $path = IMAGE_PATH . '/' . APP_MODE . '/' . $attachment_data['table_name'] . '/' . $attachment_data['table_row_id'] . '/';
        $response_file_upload = $this->imageUploadByForm($attachment_data, $request_data['file'], $path, false);
        if ($response_file_upload['status'] == RESPONSE_STATUS_ERROR) {
            return ['status' => RESPONSE_STATUS_ERROR, 'message' => __('Error when save attachment.')];
        }
        return $response_file_upload;
    }
}
