<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Prunable;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;

class NotificationEvent extends AppModel
{
    use HasFactory, Prunable, SoftDeletes;
    protected $connection = 'mysql';
    protected $with = [];
    protected $attributes = ['id' => 0];
    protected $guarded = ['type'];
    protected $dateFormat = 'Y-m-d H:i:s';
    protected $dates = [];
    public static function rules($id = null)
    {
        return [
            'notification_type_id' => 'required',
            'name' => [
                'required',
                'string',
                'max:191',
                Rule::unique('notification_types', 'name')->ignore($id),
            ],
        ];
    }

    protected $casts = [
        'created_at' => 'date:d-m-Y',
        'updated_at' => 'date:d-m-Y',
        'entity_type_role_ids' => 'array', // convert array to json and json to array while save and retrive
    ];

    public function prunable(): Builder
    {
        return static::where('id', '=', -100000000);
    }

    private function prepareSearchQuery($query, $request)
    {
        $search_keys = ['id','notification_type_id','status'];
        foreach ($search_keys as $key) {
            if (isset($request[$key]) && intval($request[$key]) > 0) {
                $query->whereIn($key, is_array(($request[$key])) ? $request[$key] : [$request[$key]]);
            }
        }

        if (isset($request['search_string']) && strlen($request['search_string']) > 0) {
            $query->where('title', 'like', "%" . $request['search_string'] . "%");
        }
        return $query;
    }

    private function eventDeletionRoles($query)
    {
        return true;
    }


    public function notificationType()
    {
        return $this->belongsTo(NotificationType::class, 'notification_type_id');
    }

    public function findByName($name)
    {
        return $this->where('name', $name)->first();
    }

    public function getEvents($user_id)
    {
        $notifications = [];
        $m_user_entity_role = new UserEntityRole();
        $user_entity_role = $m_user_entity_role->findByUserId($user_id);
        if ($user_entity_role) {
            //query role id check in json
//            $notifications = $this->whereRaw('entity_type_role_ids LIKE :json', ['json' => '%"'.$user_entity_role->entity_type_role_id.'"%'])->get();
            $notifications = $this->all();
        }
        return $notifications;
    }

    public function getList($page, $limit, $start, $search)
    {
        $query = $this::with('notificationType')->select(['id','notification_type_id','name','title','description','entity_type_role_ids', 'status']);
        $query = $this->prepareSearchQuery($query, ['search_string' => $search['value']]);
        $_total = $query->count();
        $notificationEvents = $query->limit($limit)->offset($start)->get()->toArray();
        $final_data = [];
        $i = 0;
        foreach ($notificationEvents as $notificationEvent) {
            $index = (++$i) + $start;
            $row = [
                'sl' => $index,
                'id' => $notificationEvent['id'],
                'notification_type_id' => $notificationEvent['notification_type']['type_name'] ?? '',
                'name' => $notificationEvent['name'],
                'title' => $notificationEvent['title'],
                'description' => $notificationEvent['description'],
                'entity_type_role_ids' => $notificationEvent['entity_type_role_ids'],
                'status' => $notificationEvent['status'],
                'action' => '',
            ];
            $final_data[] = $row;
        }
        return ['draw' => $page, 'recordsTotal' => $_total, 'recordsFiltered' => $_total, 'data' => $final_data];
    }

    public function saveOrUpdate(array $data)
    {
        $entity_type_role_ids = null;
        if (isset($data['user_role']) && isset($data['message'])) {
            $userRoles = $data['user_role'];
            $messages = $data['message'];
            $transformedData = [];
            for ($i = 0; $i < count($userRoles); $i++) {
                if (!empty($userRoles[$i]) && isset($messages[$i])) {
                    $transformedData[$userRoles[$i]] = $messages[$i];
                }
            }
            $entity_type_role_ids = $transformedData;
        }
        $data['entity_type_role_ids'] = $entity_type_role_ids;
        return $this->modifiedSave($data, []);
    }

    public function deleteNotificationEvent($id)
    {
        $query = $this->findOrFail($id);
        $delete_role = $this->eventDeletionRoles($query);
        if ($delete_role === true) {
            $query->delete();
            return ['message' => __('Deleted Successfully'), 'status' => 'success'];
        } else {
            return ['message' => __($delete_role), 'status' => 'error'];
        }
    }

    public function getOptions($request = null)
    {
        try {
            $query = $this::select('id', 'name');
            return $this->prepareSearchQuery($query, ['search_string' => $request['value']])->pluck('name','id');
        } catch (\Exception $e) {
            return [];
        }
    }
}
