<?php

namespace App\Models;

use App\Helpers\APIHelper;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Prunable;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Http;

class NotificationMessage extends AppModel
{
    use HasFactory, Prunable, SoftDeletes;
    protected $connection = 'mysql';
    protected $with = [];
    protected $attributes = ['id' => 0];
    protected $guarded = ['type'];
    protected $dateFormat = 'Y-m-d H:i:s';
    protected $dates = ['notification_sent_at', 'notification_last_updated_at'];
    public static function rules()
    {
        return [
            'sender_user_id' => 'required',
            'receiver_user_id' => 'required',
        ];
    }

    protected $casts = [
        'notification_sent_at' => 'date:d-M-Y h:i A',
        'notification_last_updated_at' => 'date:d-M-Y',
        'created_at' => 'date:d-M-Y',
        'updated_at' => 'date:d-M-Y',
    ];

    public function prunable(): Builder
    {
        return static::where('id', '=', -100000000);
    }

    private function prepareSearchQuery($query, $request)
    {
        $search_keys = ['id', 'client_profile_id', 'sender_profile_id', 'sender_user_id', 'receiver_profile_id', 'receiver_user_id', 'notification_type_id'];
        foreach ($search_keys as $key) {
            if (isset($request[$key]) && intval($request[$key]) > 0) {
                $query->whereIn($key, is_array(($request[$key])) ? $request[$key] : [$request[$key]]);
            }
        }

        if (isset($request['search_string']) && strlen($request['search_string']) > 0) {
            $searchValue = $request['search_string'];
            $query->where(function ($subquery) use ($searchValue) {
                $subquery->where('notification_message', 'like', '%' . $searchValue . '%')
                    ->orWhereHas('notificationType', function ($subquery) use ($searchValue) {
                        $subquery->where('type_name', 'like', '%' . $searchValue . '%');
                    });
            });
        }
        return $query;
    }

    private function messageDeletionRoles($query)
    {
        return true;
    }

    public function senderProfile()
    {
        return $this->belongsTo(UserProfile::class, 'sender_profile_id');
    }

    public function senderUser()
    {
        return $this->belongsTo(User::class, 'sender_user_id');
    }

    public function receiverProfile()
    {
        return $this->belongsTo(UserProfile::class, 'receiver_profile_id');
    }

    public function receiverUser()
    {
        return $this->belongsTo(User::class, 'receiver_user_id');
    }

    public function notificationType()
    {
        return $this->belongsTo(NotificationType::class, 'notification_type_id');
    }

    public function getNotificatonMessage($id)
    {
        return $this->findOrFail($id);
    }

    public function getList($page, $limit, $start, $search, $auth_user)
    {
        $search_data['receiver_user_id'] =  0;
        if (isset($search['profile_id']) && $search['profile_id'] > 0) {
            $m_up = new UserProfile();
            $profile = $m_up->getProfile($search['profile_id']);
            $search_data['receiver_user_id'] = $profile['user_id'] ?? 0;
        } else {
            $search_data['receiver_user_id'] = $auth_user['current_role'] == ROLE_ADMIN ? 0 :  $auth_user['id'];
        }
        $search_data['client_profile_id'] = $search['client_id'] ?? 0;
        $search_data['search_string'] = $search['value'];
        if (DATA_WATCH_ENABLE) {
            return $this->dataWatchGetList($page, $limit, $start, $search_data);
        }
        $query = $this::with('senderProfile', 'receiverProfile', 'senderUser.role.entity_type', 'receiverUser.role.entity_type', 'notificationType')->select(['id', 'sender_profile_id', 'sender_user_id', 'receiver_profile_id', 'receiver_user_id', 'client_profile_id', 'notification_type_id', 'notification_message', 'notification_sent_at']);
        $query = $this->prepareSearchQuery($query, $search_data);
        $_total = $query->count();
        $notifications = $query->limit($limit)->offset($start)->get()->toArray();
        $final_data = [];
        $i = 0;
        foreach ($notifications as $notification) {
            $index = (++$i) + $start;
            $sender_name = $notification['sender_profile']['full_name'] ?? '';
            $sender_role = $notification['sender_user']['role']['entity_type']['role_name'] ?? '';
            $receiver_name = $notification['receiver_profile']['full_name'] ?? '';
            $receiver_role = $notification['receiver_user']['role']['entity_type']['role_name'] ?? '';
            $row = [
                'sl' => $index,
                'id' => $notification['id'],
                'type' => $notification['notification_type']['type_description'] ?? '',
                'message' => $notification['notification_message'],
                'sender' => '<div class="d-flex align-items-center flex-row-fluid flex-wrap">
                    <div class="flex-grow-1 me-2">
                        <a href="javascript:void(0)"
                            class="text-dark text-hover-primary fs-6 ">' . $sender_name . '</a>
                        <span class="text-muted fw-semibold fs-7">(' . strtolower($sender_role) . ')</span>
                    </div></div>',
                'time' =>  $notification['notification_sent_at'],
            ];
            if (!empty($search_data['client_profile_id']) && $search_data['client_profile_id'] > 0) {
                $row['receiver'] = '<div class="d-flex align-items-center flex-row-fluid flex-wrap">
                    <div class="flex-grow-1 me-2">
                        <a href="javascript:void(0)"
                            class="text-dark text-hover-primary fs-6 ">' . $receiver_name . '</a>
                        <span class="text-muted fw-semibold fs-7">(' . strtolower($receiver_role) . ')</span>
                    </div></div>';
            }
            $final_data[] = $row;
        }
        return ['draw' => $page, 'recordsTotal' => $_total, 'recordsFiltered' => $_total, 'data' => $final_data];
    }

    public function deleteNotificationMessage($id)
    {
        $query = $this->findOrFail($id);
        $delete_role = $this->messageDeletionRoles($query);
        if ($delete_role === true) {
            $query->delete();
            return ['message' => __('Deleted Successfully'), 'status' => 'success'];
        } else {
            return ['message' => __($delete_role), 'status' => 'error'];
        }
    }

    public function saveOrUpdate(array $data)
    {
        return $this->modifiedSave($data, []);
    }

    public function saveNotificationDataPrepare($user_id, $event_id, $message, $url, $client_id = 0)
    {
        $user = new User();
        $user = $user->find($user_id);
        if ($user) {
            if ($client_id == 0) {
                $client_id = $user->userEntity->entity_profile_id ?? 0;
            }
            $m_notification_event = new NotificationEvent();
            $m_notification_event = $m_notification_event->find($event_id);
            $data = [
                'client_profile_id' => $client_id,
                'sender_profile_id' => auth()->user()->profile->id,
                'sender_user_id' => auth()->id(),
                'receiver_profile_id' => $user->profile->id,
                'receiver_user_id' => $user_id,
                'notification_status' => 1,
                'notification_sent_at' => now(),
                'notification_last_updated_at' => now(),
                'notification_type_id' => $m_notification_event->notification_type_id,
                'notification_title' => $m_notification_event->title,
                'notification_message' => $message,
                'notification_action_url' => $url,
                'notification_action_text' => $m_notification_event->name,
            ];
            $response =  $this->saveOrUpdate(setAuthInfoToData($data, Auth::user()));
            if ($response['status'] == "success") {
                return true;
            }
        }
        return false;
    }

    public function getPushNotificationList($page, $limit, $start, $search)
    {
        $user_id = $search['user_id'] ?? auth()->id();
        $response = Http::withoutVerifying()->get(GET_NOTIFICATION_URL . "?publisher_id=" . config('push_notify.username') . "&subscriber=" . config('push_notify.username') . "_" . $user_id . "&page=" . $page . "&per_page=" . $limit);
        $final_data = [];
        $_total = 0;
        if ($response->successful()) {
            $response_data = $response->json();
            if (isset($response_data['status']) && $response_data['status'] == 'success') {
                $notifications = $response_data['data'];
                $_total = count($notifications);
                $i = 0;
                foreach ($notifications as $notification) {
                    $body = json_decode($notification['message'], true);
                    $index = (++$i) + $start;
                    $row = [
                        'sl' => $index,
                        'id' => $notification['id'],
                        'click' => $notification['click'],
                        'title' => $notification['title'],
                        'time' => $notification['time'],
                        'message' => $body['content'],
                    ];
                    $final_data[] = $row;
                }
            }
        }
        return ['draw' => $page, 'recordsTotal' => $_total, 'recordsFiltered' => $_total, 'data' => $final_data];
    }

    public function getNotificationMessage($page, $limit, $start, $search)
    {
        $user_id = $search['user_id'] ?? auth()->id();
        if (DATA_WATCH_ENABLE) {
            $search_keys['receiver_user_id'] = $user_id;
            return $this->getNotificationDataWatch($page, $limit, $start, $search_keys);
        } else {
            return $this->getNotificationList($page, $limit, $start, $user_id);
        }
    }

    public function getNotificationList($page, $limit, $start, $user_id)
    {
        $query = $this->select(['*']);
        $messages = $this->prepareSearchQuery($query, ['receiver_user_id' => $user_id])
            ->limit($limit)
            ->offset($start)
            ->latest('id')
            ->get()
            ->toArray();

        $final_data = [];
        $_total = count($messages);
        $i = 0;
        foreach ($messages as $message) {
            $index = (++$i) + $start;
            $row = [
                'sl' => $index,
                'id' => $message['id'],
                'click' => $message['notification_action_url'],
                'title' => $message['notification_title'],
                'time' => strtotime($message['notification_sent_at']),
                'message' => $message['notification_message'],
                'view_status' => $message['view_status'],
            ];
            $final_data[] = $row;
        }
        return ['draw' => $page, 'recordsTotal' => $_total, 'recordsFiltered' => $_total, 'data' => $final_data];
    }

    // ** START DATA-WATCH NOTIFICATION **
    public function saveDataWatchNotification($user_id, $event_id, $message, $url, $client_id = 0, $client_name = '')
    {
        $m_user = new User();
        $user = $m_user->userDetails($user_id);
        if ($user) {
            if ($client_id == 0) {
                $client_id = $user->userEntity->entity_profile_id ?? 0;
            }
            $m_notification_event = new NotificationEvent();
            $m_notification_event = $m_notification_event->find($event_id);

            $auth_user = auth()->user();
            $auth_profile = $auth_user->profile;
            $sender_user_id = $auth_user->id;
            $sender_profile_id = $auth_profile->id;
            $sender_profile_name = $auth_profile->full_name;
            $sender_role = strtolower($auth_user->role->entity_type->role_name);

            $receiver_profile_id = $user->profile->id;
            $receiver_profile_name = $user->profile->full_name;
            $receiver_role = strtolower($user->role->entity_type->role_name);

            $data = [
                'client_profile_id' => $client_id,
                'client_name' => $client_name,
                'sender_profile_id' => $sender_profile_id,
                'sender_profile_name' => $sender_profile_name,
                'sender_role' => $sender_role,
                'sender_user_id' => $sender_user_id,
                'receiver_profile_id' => $receiver_profile_id,
                'receiver_profile_name' => $receiver_profile_name,
                'receiver_profile_role' => $receiver_role,
                'receiver_user_id' => $user_id,
                'notification_status' => 1,
                'notification_sent_at' => now(),
                'notification_last_updated_at' => now(),
                'notification_type_id' => $m_notification_event->notification_type_id,
                'notification_type_name' => $m_notification_event->notificationType->type_name ?? '',
                'notification_title' => $m_notification_event->title,
                'notification_message' => $message,
                'notification_action_url' => $url,
                'notification_action_text' => $m_notification_event->name,
                'view_status' => 'new',
                'meta_data' => [
                    'client_profile_id' => $client_id,
                    'client_name' => $client_name,
                    'sender_profile_id' => $sender_profile_id,
                    'sender_profile_name' => $sender_profile_name,
                    'sender_role' => $sender_role,
                    'sender_user_id' => $sender_user_id,
                    'receiver_profile_id' => $receiver_profile_id,
                    'receiver_profile_name' => $receiver_profile_name,
                    'receiver_profile_role' => $receiver_role,
                    'receiver_user_id' => $user_id,
                ]
            ];
            $path = PUSH_NOTIFICATON_SAVE_API;
            return $this->sendToDataWatch($data, $path);
        }
        return false;
    }

    public function sendToDataWatch($data, $path)
    {
        try {
            $url = DATA_WATCH_URL . $path;
            $headers = ['Content-Type: application/json'];
            $notification_data = [
                'index' => APP_MODE . '_' . PUSH_NOTIFICATION_DATA_WATCH_INDEX,
                'notification_data' => json_encode($data),
                'request_url' => APP_NAME
            ];
            $apiHelper = new APIHelper();
            $response = $apiHelper->callAPI($url, $headers, 'POST', json_encode($notification_data));
            if ($response['status'] == RESPONSE_STATUS_SUCCESS) {
                return true;
            } else {
                return false;
            }
        } catch (\Exception $ex) {
            return false;
        }
    }

    public function getNotificationDataWatch($page, $limit, $start, $search_keys, $from = '', $to = '')
    {
        try {
            $url = DATA_WATCH_URL . GET_PUSH_NOTIFICATION_API;
            $headers = ['Content-Type: application/json'];
            $start = ($start / $limit) + 1;
            // Assign validated values from $request to $data array
            $data = [
                'index' => APP_MODE . '_' . PUSH_NOTIFICATION_DATA_WATCH_INDEX,
                'length' => $limit,
                'page' => $start,
                'order' => 'desc',
                'from' => $from,
                'to' => $to,
                'search_keyword' => json_encode([
                    'receiver_user_id' => $search_keys['receiver_user_id'] ?? 0,
                    'client_profile_id' => $search_keys['client_profile_id'] ?? 0
                ]),
            ];
            $apiHelper = new APIHelper();
            $messages = $apiHelper->callAPI($url, $headers, 'POST', json_encode($data));
            $final_data = [];
            $_total = count($messages['data']);
            $i = 0;
            foreach ($messages['data'] as $message) {
                $index = (++$i) + $start;
                $row = [
                    'sl' => $index,
                    'id' => $message['id'],
                    'click' => $message['notification_data']['notification_action_url'],
                    'title' => $message['notification_data']['notification_title'],
                    'time' => strtotime($message['notification_data']['notification_sent_at']),
                    'message' => $message['notification_data']['notification_message'],
                    'view_status' => $message['notification_data']['view_status'],
                ];
                $final_data[] = $row;
            }
            return ['draw' => $page, 'recordsTotal' => $_total, 'recordsFiltered' => $_total, 'data' => $final_data];
        } catch (\Exception $ex) {
            return false;
        }
    }

    public function dataWatchGetList($page, $limit, $start, $search_data)
    {
        try {
            $url = DATA_WATCH_URL . GET_PUSH_NOTIFICATION_API;
            $headers = ['Content-Type: application/json'];
            // Assign validated values from $request to $data array
            $data = [
                'index' => APP_MODE . '_' . PUSH_NOTIFICATION_DATA_WATCH_INDEX,
                'length' => $limit,
                'page' => ($start / $limit) + 1,
                'order' => 'desc',
                'search_keyword' => json_encode([
                    'receiver_user_id' => $search_data['receiver_user_id'] ?? 0,
                    'client_profile_id' => $search_data['client_profile_id'] ?? 0
                ]),
            ];
            $apiHelper = new APIHelper();
            $notifications = $apiHelper->callAPI($url, $headers, 'POST', json_encode($data));
            $_total = $notifications['total_row'];
            $final_data = [];
            $i = 0;
            foreach ($notifications['data'] as $notification) {
                $index = (++$i) + $start;
                $sender_name = $notification['notification_data']['meta_data']['sender_profile_name'] ?? '';
                $sender_role = $notification['notification_data']['meta_data']['sender_role'] ?? '';
                $receiver_name = $notification['notification_data']['meta_data']['receiver_profile_name'] ?? '';
                $receiver_role = $notification['notification_data']['meta_data']['receiver_profile_role'] ?? '';
                $row = [
                    'sl' => $index,
                    'id' => $notification['id'],
                    'type' => $notification['notification_data']['notification_type_name'] ?? '',
                    'message' => $notification['notification_data']['notification_message'],
                    'sender' => '<div class="d-flex align-items-center flex-row-fluid flex-wrap">
                    <div class="flex-grow-1 me-2">
                        <a href="javascript:void(0)"
                            class="text-dark text-hover-primary fs-6 ">' . $sender_name . '</a>
                        <span class="text-muted fw-semibold fs-7">(' . strtolower($sender_role) . ')</span>
                    </div></div>',
                    'time' =>  date('d-M-Y h:i A', strtotime($notification['notification_data']['notification_sent_at'])),
                ];
                // if (!empty($search_data['client_profile_id']) && $search_data['client_profile_id'] > 0) {
                    $row['receiver'] = '<div class="d-flex align-items-center flex-row-fluid flex-wrap">
                    <div class="flex-grow-1 me-2">
                        <a href="javascript:void(0)"
                            class="text-dark text-hover-primary fs-6 ">' . $receiver_name . '</a>
                        <span class="text-muted fw-semibold fs-7">(' . strtolower($receiver_role) . ')</span>
                    </div></div>';
                // }
                $final_data[] = $row;
            }
            return ['draw' => $page, 'recordsTotal' => $_total, 'recordsFiltered' => $_total, 'data' => $final_data];
        } catch (\Exception $ex) {
            return false;
        }
    }

    public function seenNotification(array $request_data)
    {
        try {
            $data['id'] = $request_data['id'];
            $data['view_status'] = 'view';
            $path = PUSH_NOTIFICATON_STATUS_UPDATE_API;
            $response = $this->sendToDataWatch($data, $path);
            if ($response) {
                return ['status' => RESPONSE_STATUS_SUCCESS, 'message' => __('Read successfully.')];
            }
        } catch (\Exception $e) {
            return ['status' => RESPONSE_STATUS_ERROR, 'message' => $e->getMessage()];
        }
    }
    // ** END DATA-WATCH NOTIFICATION **
}
