<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Prunable;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Laravel\Sanctum\HasApiTokens;
use Tymon\JWTAuth\Contracts\JWTSubject;

class User extends Authenticatable implements JWTSubject
{
    use HasApiTokens, HasFactory, Notifiable, Prunable, SoftDeletes;
    protected $connection = 'mysql';
    protected $with = [];
    protected $attributes = ['id' => 0];
    protected $guarded = ['type'];
    protected $dateFormat = 'Y-m-d H:i:s';
    public function rules()
    {
        return [
            'role_id' => 'required',
            'username' => 'required|unique:users',
            'email' => 'required',
            'new_password' => 'required',
            'confirm_password' => 'required'
        ];
    }
    protected $casts = [
        'email_verified_at' => 'datetime',
        'password' => 'hashed',
    ];
    public function prunable()
    {
        return static::where('id', '=', -100000000);
    }
    protected $hidden = [
        // 'password',
        'remember_token',
    ];
    public function getJWTIdentifier()
    {
        return $this->getKey();
    }
    public function getJWTCustomClaims()
    {
        return [
            'username' => $this->username,
        ];
    }
    public function getProfileInfoAttribute()
    {
        return $this->profile;
    }
    /**** Relation: Start */
    public function profile()
    {
        return $this->hasOne(UserProfile::class, 'user_id', 'id');
    }
    public function role()
    {
        return $this->hasOne(UserEntityRole::class);
    }
    /**** Relation: End */
    /**** Private Functions: Starts */
    function prepareSearchQuery($query, $request)
    {
        $search_keys = ['id', 'user_id'];
        foreach ($search_keys as $key) {
            if (isset($request[$key])) {
                $query->whereIn($key, is_array(($request[$key])) ? $request[$key] : [$request[$key]]);
            }
        }

        if (isset($request['client_profile_id']) && intval($request['client_profile_id']) > 0) {
            $m_user_er = new UserEntityRole();
            $user_ids = $m_user_er->clientWiseTotalUsers($request['client_profile_id'])->pluck('user_id', 'user_id');
            $query->whereIn('id', $user_ids);
        }

        if (isset($request['search_string']) && strlen($request['search_string']) > 0) {
            $searchTerm = $request['search_string'];
            $query->whereHas('profile', function ($query) use ($searchTerm) {
                $query->where('name_title', 'like', '%' . $searchTerm . '%')
                    ->orWhere('first_name', 'like', '%' . $searchTerm . '%')
                    ->orWhere('last_name', 'like', '%' . $searchTerm . '%')
                    ->orWhere('designation', 'like', '%' . $searchTerm . '%')
                    ->orWhere('employee_code', 'like', '%' . $searchTerm . '%')
                    ->orWhere('mobile', 'like', '%' . $searchTerm . '%')
                    ->orWhere('email', 'like', '%' . $searchTerm . '%');
            });
        }
        return $query;
    }

    public function modifiedSave($requestdata, $validation_rules)
    {
        try {
            if (!array_key_exists('id', $requestdata)) {
                throw new \Exception('Invalid request!', 403);
            }
            if (!empty($validation_rules)) {
                $validator = Validator::make($requestdata, $validation_rules);
                if ($validator->fails()) {
                    return [
                        'status' => 'error',
                        'code' => 422,
                        'errors' => $validator->errors(),
                        'message' => __("Validation Error Occured")
                    ];
                }
            }
            $content = $this->updateOrCreate(['id' => $requestdata['id']], $requestdata);
            return ['status' => 'success', 'code' => 200, 'data' => $content, 'id' => $content->id, 'message' => "Saved successfully"];
        } catch (\Exception $e) {
            return ['status' => 'error', 'code' => 500, 'errorModel' => '', 'message' => $e->getMessage()];
        }
    }
    /**** public functions: Starts */
    public function saveOrUpdate($data)
    {
        return $this->modifiedSave($data, []);
    }

    public function getOptions($options = [])
    {
        $_data_options = $this->all()->pluck('id', 'id');
        return $_data_options;
    }
    //Formatting Ends

    public function changePassword(array $data)
    {
        try {
            if (empty($data['user_id']) || empty($data['new_password'])) {
                throw new \Exception('Invalid request!');
            }
            if (array_key_exists('current_password', $data) && empty($data['current_password'])) {
                throw new \Exception('Current password required!');
            }
            $userEntity = $this->findOrFail($data['user_id']);
            if (array_key_exists('current_password', $data) && !empty($data['current_password']) && Hash::check($data['current_password'], $userEntity['password']) == false) {
                throw new \Exception('Current password not matched!');
            }
            if ($data['confirm_password'] === $data['new_password']) {
                $userEntity['password'] = $data['new_password'];
                $userEntity['force_change_pass'] = 0;
                $userEntity->save();
            } else {
                throw new \Exception('Current password and new password not matched!');
            }
        } catch (\Exception $ex) {
            return [
                'status' => 'error',
                'message' => $ex->getMessage()
            ];
        }
        return [
            'status' => 'success',
            'data' => $userEntity->toArray()
        ];
    }
    public function changeUsername(array $data)
    {
        try {
            if (empty($data['user_id']) || empty($data['username'])) {
                throw new \Exception('Invalid request!');
            }
            $userEntity = $this->findOrFail($data['user_id']);
            if ($data['is_admin'] == false) {
                if (Hash::check($data['password'], $userEntity['password']) == false) {
                    throw new \Exception('Current password not matched!');
                }
            }
            $userEntity['username'] = $data['username'];
            $userEntity->save();
        } catch (\Exception $ex) {
            return [
                'status' => 'error',
                'message' => $ex->getMessage()
            ];
        }
        return [
            'status' => 'success',
            'data' => $userEntity
        ];
    }
    public function statusUpdate(array $data)
    {
        try {
            if (empty($data['uid'])) {
                throw new \Exception('Invalid request!');
            }
            $m_userProfile = new UserProfile();
            $m_userLog = new UserLog();

            $profile = $m_userProfile->getProfile($data['uid']);
            $user = $this->userDetails($profile->user_id);
            $newStatus = ($user->active_status == 1) ? 0 : 1;

            $logData = [
                'user_id' => $user->id,
                'log_type' => ($newStatus == 1) ? 'user_active' : 'user_inactive',
                'log_data' => json_encode([
                    'active_status' => [
                        'oldvalue' => $user->active_status,
                        'newvalue' => $newStatus,
                    ],
                ]),
                'user_data' => json_encode([
                    'name' => $user->profile ? $user->profile->name : '',
                    'username' => $user->username,
                ]),
            ];

            // Create a user log entry
            if ($m_userLog->logCreate($logData)) {
                // Update profile and user active status
                if ($user->profile) {
                    $profileData = [
                        'id' => $profile->id,
                        'active_status' => $newStatus,
                    ];
                    $m_userProfile->saveOrUpdate(setAuthInfoToData($profileData, Auth::user()));
                }

                $userData = [
                    'id' => $user->id,
                    'active_status' => $newStatus,
                ];
                $this->saveOrUpdate(setAuthInfoToData($userData, Auth::user()));
            } else {
                throw new \Exception('Something went wrong!');
            }
            return [
                'status' => 'success',
                'data' => $user,
            ];
        } catch (\Exception $ex) {
            return [
                'status' => 'error',
                'message' => $ex->getMessage(),
            ];
        }
    }
    public function userDelete(array $data)
    {
        try {
            if (empty($data['user_id'])) {
                throw new \Exception('Invalid request!');
            }
            $userEntity = $this->findOrFail($data['user_id']);
            $log_data_decode['users'] = $userEntity->toArray();
            $log_data_decode['user_profiles'] = $userEntity->profile->toArray();
            $log_data_decode['user_entity_roles'] = $userEntity->userEntity->toArray();
            $log = new UserLog();
            $log_data['user_id'] = $userEntity->id;
            $log_data['log_type'] = 'user delete';
            $log_data['log_data'] = json_encode($log_data_decode);
            $log_data['user_data'] = json_encode([
                'name' => $userEntity->profile ? $userEntity->profile->name : "",
                'username' => $userEntity->username,
            ]);
            if ($log->logCreate($log_data)) {
                // $userEntity->profile->delete();
                // $userEntity->userEntity->delete();
                // $userEntity->delete();
            } else {
                throw new \Exception('Something went wrong!!');
            }
        } catch (\Exception $ex) {
            return [
                'status' => 'error',
                'message' => $ex->getMessage()
            ];
        }
        return [
            'status' => 'success',
            'data' => []
        ];
    }
    public function getUserByCreator($creator_id)
    {
        return User::where('created_by', $creator_id)
            ->with('profile')
            ->get()
            ->map(function ($user) {
                $value = $user->profile->name . '( ' . $user->username . ' )';
                return [
                    'id' => $user->id,
                    'value' => $value
                ];
            })
            ->pluck('value', 'id')
            ->toArray();
    }

    public function checkAndSendEmail($rd)
    {
        try {
            $email = $rd['email'];
            $username = $rd['username'];
            $user = $this->with('profile')->whereHas('profile', function ($query) use ($email) {
                $query->where('email', $email);
            })->where('username', $username)->first();
            if (!empty($user)) {
                $activation_code =  sprintf("%06d", mt_rand(0, 999999));
                // Create an array with the data to be saved
                $password_reset_data = [
                    'id' => 0,
                    'user_id' => $user->id,
                    'username' => $user->username,
                    'email' => $email,
                    'activation_code' => $activation_code,
                    'is_activated' => 1,
                    'expiry' => ACTIVATION_CODE_TIME_COUNT,
                    'created_by' => $user->id,
                    'modified_by' => $user->id,
                ];
                // Save the entity
                $m_user_activation_code = new UserActivationCode();
                $response_user_eactivation_code = $m_user_activation_code->saveOrUpdate($password_reset_data);
                if ($response_user_eactivation_code == RESPONSE_STATUS_ERROR) {
                    return $response_user_eactivation_code;
                }
                $body = __('Your Verification Code is: ') . $activation_code;
                $userActivationCode = new UserLog();
                $userActivationCode->passwordResetLog($user);

                $mailServiceTable = new EmailService();
                $response_mail = $mailServiceTable->sendEmail(__('Forget Password'), $body, $user->profile->full_name, $email, [], $user->id);
                if ($response_mail['status'] == RESPONSE_STATUS_ERROR) {
                    return $response_mail;
                }
                $expiry = convert_to_seconds(ACTIVATION_CODE_TIME_COUNT . ' ' . ACTIVATION_CODE_TIME_FORMAT);
                return ['status' => RESPONSE_STATUS_SUCCESS, 'message' => __('Password is sent to your email'), 'expiry' => $expiry];
            }
            return ['status' => RESPONSE_STATUS_ERROR, 'message' => __("There’s no account with the info you provided.")];
        } catch (\Exception $ex) {
            return ['status' => RESPONSE_STATUS_ERROR, 'message' => $ex->getMessage()];
        }
    }

    public function verifyEmailCode($rd)
    {
        try {
            $email = $rd['email'];
            $username = $rd['username'];
            $activation_code = $rd['verification_code'];

            $m_activation_code = new UserActivationCode();
            $response_activation_code = $m_activation_code->getUserActivationData(['email' => $email, 'username' => $username])->orderBy('created_at', 'DESC')->first();
            if (empty($response_activation_code)) {
                return ['status' => RESPONSE_STATUS_ERROR, 'message' => __('Invalid request!')];
            }
            if ((Carbon::parse($response_activation_code->created_at)->lessThan(Carbon::now()->subMinutes($response_activation_code->expiry)) == false) &&  $response_activation_code->activation_code === $activation_code) {
                $user = $this->whereHas('profile', function ($query) use ($email) {
                    $query->where('email', $email);
                })->where('username', $username)->first();

                if (empty($user)) {
                    return ['status' => RESPONSE_STATUS_ERROR, 'message' => __('User not found!')];
                } else {
                    $user['password'] = DEFAULT_PASSWORD;
                    $user['force_change_pass'] = 1;
                    if ($user->save()) {
                        return ['status' => RESPONSE_STATUS_SUCCESS, 'message' => __('Successfully saved1'), 'data' => ['username' => $user->username]];
                    }
                }
            } else {
                return ['status' => RESPONSE_STATUS_ERROR, 'message' => __('Invalid request!')];
            }
        } catch (\Exception $e) {
            return ['status' => RESPONSE_STATUS_ERROR, 'message' => $e->getMessage()];
        }
    }

    public function getUserbyDefaultPass($data)
    {
        $username = isset($data['username']) ? $data['username'] : '';
        $password = DEFAULT_PASSWORD;
        $user = $this->where('username', $username)->first();
        if ($user && password_verify($password, $user->password)) {
            return $user;
        } else {
            return null;
        }
    }

    public function saveUser($data, $send_mail = true)
    {
        $response = ['status' => 'error', 'code' => 500, 'message' => __('An error occurred while saving user.')];
        try {
            $m_user = $this;
            if (!empty($this->validation_rules)) {
                $validator = Validator::make($data, $this->validation_rules);
                if ($validator->fails()) {
                    return [
                        'status' => 'error',
                        'code' => 422,
                        'errors' => $validator->errors(),
                        'message' => __("Validation Error Occured")
                    ];
                }
            }
            $m_user_profile = new UserProfile();
            if (!empty($m_user_profile->validation_rules)) {
                $validator = Validator::make($data, $m_user_profile->validation_rules);
                if ($validator->fails()) {
                    return [
                        'status' => 'error',
                        'code' => 422,
                        'errors' => $validator->errors(),
                        'message' => __("Validation Error Occured")
                    ];
                }
            }
            $return = DB::transaction(function () use (&$response, $m_user, $m_user_profile, &$data) {
                $user = $m_user->create($data);
                if (empty($user)) {
                    $response = ['status' => 'error', 'code' => 500, 'message' => __("Failed to save user")];
                    return false;
                }
                $m_user_profile = new UserProfile();
                $data['user_id'] = $user['id'];
                $response = $m_user_profile->modifiedSave($data, []);
                if ($response['status'] != 'success') {
                    return false;
                }
                $user_profile_id = $response['data']['id'];
                $data['user_full_name'] = $response['data']['full_name'];
                if (!empty($data['profile_image']) || !empty($data['signature_image'])) {
                    $file_attachment = new FileAttachment();
                    $send_data['table_name'] = USER_PROFILE_TABLE_NAME;
                    $send_data['table_row_id'] = $response['id'];
                    if (!empty($data['profile_image'])) {
                        $send_data['image'] = $data['profile_image'];
                        $send_data['original_name'] = isset($data['profile_image_name']) ? $data['profile_image_name'] : '';
                        $send_data['x_attachment_type_id'] = USER_PROFILE_ATTACHMENT_TYPE_KEY;
                        $f_response = $file_attachment->base64Upload($send_data);
                    }
                    if (!empty($data['signature_image'])) {
                        $send_data['image'] = $data['signature_image'];
                        $send_data['original_name'] = isset($data['signature_image_name']) ? $data['signature_image_name'] : '';
                        $send_data['x_attachment_type_id'] = USER_SIGNATURE_ATTACHMENT_TYPE_KEY;
                        $f_response = $file_attachment->base64Upload($send_data);
                    }
                }
                $user_entity_role = new UserEntityRole();
                $data['user_profile_id'] = $user_profile_id;
                if (isset($data['supervisor']) && !empty($data['supervisor'])) {
                    $data['supervisor_user_profile_id'] = $data['supervisor']['id'];
                    $data['supervisor_user_id'] = $data['supervisor']['user_id'];
                }
                $data['entity_profile_id'] = isset($data['entity_profile_id']) ? $data['entity_profile_id'] : 0;
                $data['is_default_role'] = isset($data['is_default_role']) ? $data['is_default_role'] : 1;
                $data['is_admin'] = isset($data['is_admin']) ? intval($data['is_admin']) : 0;
                $role_response = $user_entity_role->saveUserEntityRole($data);
                if ($role_response['status'] != 'success') {
                    $response = $role_response; //to show the correct error
                    return false;
                }
                $response['entity_profile_id'] = $role_response['data']['entity_profile_id'];
                return true;
            });
            if ($return && !isset($data['client_create'])) {
                if (!isset($data['user_full_name'])) {
                    dd($data);
                }
                if ($send_mail) {
                    $this->sendUserCreationMail($data['user_full_name'], $data['email'], $data['username'], $data['password']);
                }
            }
        } catch (\Exception $e) {
            $response = ['status' => 'error', 'code' => 500, 'message' => __('An error occurred while saving user.'), 'error' => $e->getMessage()];
        }
        return $response;
    }

    public function sendUserCreationMail($name, $email, $username, $password = DEFAULT_PASSWORD)
    {
        $m_email_template = new EmailTemplate();
        $body = $m_email_template->loginCredentialsTemplate($name, $username, $password);
        $mailServiceTable = new EmailService();
        return  $mailServiceTable->sendEmail('Login Credentials', $body, $username, $email);
    }

    public function userDetails($id)
    {
        return $this->find($id)->with('profile', 'accounting_firm_profile', 'role.entity_type');
    }

    public function getUser($condition, $select = ['*'])
    {
        $query = $this->select($select);
        return $this->prepareSearchQuery($query, $condition);
    }

    public function deleteUser($id)
    {
        return $this->where('id', $id)->delete();
    }

    public function createClientUser($user_data)
    {
        try {
            $exists = $this->where('username', $user_data['username'])->exists();
            if ($exists == true) {
                return ['status' => 'error', 'message' => 'Username Already Exists.'];
            }
            return $this->create($user_data);
        } catch (\Exception $ex) {
            return ['status' => 'error', 'message' => 'Geting erro when create user.'];
        }
    }

    public function roleFindByUserId($user_id)
    {
        $auth_user = $this->userDetails($user_id);
        if (strtolower($auth_user->role->entity_type->role_name) == ROLE_ADMIN && $auth_user->role->entity_type->entity_type_id == SYSTEM_ENTITY_TYPE_ID) {
            $user_role = ROLE_ADMIN;
        } elseif (strtolower($auth_user->role->entity_type->role_name) == ROLE_ADMIN && $auth_user->role->entity_type->entity_type_id != SYSTEM_ENTITY_TYPE_ID) {
            $user_role = ROLE_CLIENT;
        } else {
            $user_role = strtolower($auth_user->role->entity_type->role_name);
        }
        return $user_role;
    }

    public function accounting_firm_profile()
    {
        return $this->hasOne(MasterAccountingFirmProfile::class, 'user_id');
    }

    public function getUserSettingList($page, $limit, $start, $search)
    {
        $query = $this::select(['id', 'active_status']);
        $query = $this->prepareSearchQuery($query, ['search_string' => $search['value'] ?? ''])->with('profile', 'accounting_firm_profile', 'role.entity_type');
        $_total = $query->count();
        $users = $query->limit($limit)->offset($start)->get()->toArray();
        $final_data = [];
        $i = 0;
        foreach ($users as $user) {

            $status = $user['active_status'] == 1 ? '<span class="badge badge-light-success">Active</span>' : '<span class="badge badge-light-danger">InActive</span>';
            // $accounting_firm_profile = empty($user['accounting_firm_profile']) ? false : true;
            $accounting_firm_profile = false;
            $role_name = $user['role']['entity_type']['role_name'] ?? '-';

            $index = (++$i) + $start;
            $row = [
                'sl' => $index,
                'id' => $user['id'],
                'name' => $user['profile']['full_name'] ?? '-',
                'designation' => $user['profile']['designation'] ?? '-',
                'accounting_firm_profile' => $accounting_firm_profile,
                'accounting_firm_profile_id' => $user['accounting_firm_profile']['id'] ?? 0,
                'role' => strtolower($role_name),
                'status' => $status,
                'active_status' => $user['active_status'],
                'action' => '',
            ];
            $final_data[] = $row;
        }
        return ['draw' => $page, 'recordsTotal' => $_total, 'recordsFiltered' => $_total, 'data' => $final_data];
    }

    public function getAfUserSettingList($page, $limit, $start, $search)
    {
        $query = $this::select(['id', 'active_status']);
        $query = $this->prepareSearchQuery($query, ['search_string' => $search['value'] ?? ''])->with('profile', 'accounting_firm_profile', 'role.entity_type')->whereHas('accounting_firm_profile');
        $_total = $query->count();
        $query->orderBy('active_status', 'asc');
        $users = $query->limit($limit)->offset($start)->get()->toArray();
        $final_data = [];
        $i = 0;
        foreach ($users as $user) {

            $status = $user['active_status'] == 1 ? '<span class="badge badge-light-success">Active</span>' : '<span class="badge badge-light-danger">InActive</span>';
            $accounting_firm_profile = empty($user['accounting_firm_profile']) ? false : true;
            $role = $accounting_firm_profile === true ? __(' (Accounting Firm Profile)') : '';
            $role_name = $user['role']['entity_type']['role_name'] ?? '-';

            $index = (++$i) + $start;
            $row = [
                'sl' => $index,
                'id' => $user['id'],
                'company_name' => $user['accounting_firm_profile']['company_name'] ?? '-',
                'name' => $user['profile']['full_name'] ?? '-',
                'designation' => $user['profile']['designation'] ?? '-',
                'accounting_firm_profile' => $accounting_firm_profile,
                'accounting_firm_profile_id' => $user['accounting_firm_profile']['id'] ?? 0,
                'role' => strtolower($role_name) . $role,

                'db_setup' => '<button type="button" title="DB Setup" data-action="' . route('setup.af_profile.af_user') . '" data-key="' . $user['accounting_firm_profile']['id'] . '" id="btn_default_active_af_user" class="btn btn-info btn-active-light-info btn btn-sm rounded-0 btn_default_active_af_user">
                <i class="fa-solid fa-database"></i>
                <span class="ms-1">DB Setup</span>
                </button>',

                'dns_setup' => '<button type="button" title="DNS Setup" data-action="' . route('setup.af_profile.af_user') . '" data-key="' . $user['accounting_firm_profile']['id'] . '" id="btn_dns_setup" class="btn btn-success btn-active-light-success btn btn-sm rounded-0 btn_dns_setup">
                <i class="fa-solid fa-globe"></i>
                <span class="ms-1">DNS Setup</span>
                </button>',
                'active_status' => $user['active_status'],
                'status' => $status,
                'action' => '',
            ];
            $final_data[] = $row;
        }
        return ['draw' => $page, 'recordsTotal' => $_total, 'recordsFiltered' => $_total, 'data' => $final_data];
    }

    public function userStatusUpdate($request_data)
    {
        try {
            $status_message = $request_data['active_status'] == 0 ? __('User successfully made inactive!') : __('User successfully made active!');
            $query = $this->select('*');
            $user = $this->prepareSearchQuery($query, ['id' => $request_data['id']])->first();
            if (empty($user)) {
                return ['status' => RESPONSE_STATUS_ERROR, 'message' => __('User not found!')];
            }
            if ($user->update(['active_status' => $request_data['active_status']])) {
                $m_user_profile = new UserProfile();
                $response_user_profile = $m_user_profile->userProfileStatusUpdate($request_data['id'], $request_data['active_status']);
                if ($response_user_profile['status'] == RESPONSE_STATUS_ERROR) {
                    return $response_user_profile;
                }
                return ['status' => RESPONSE_STATUS_SUCCESS, 'message' => $status_message];
            }
        } catch (\Exception $e) {
            return ['status' => RESPONSE_STATUS_ERROR, 'message' => $e->getMessage()];
        }
    }

    ######################
    ##### user setup #####
    ######################
    public function getList($page, $limit, $start, $search)
    {
        $query = $this::select(['*']);
        $query = $this->prepareSearchQuery($query, ['search_string' => $search['value']])->with('profile', 'role.bank_branch');
        $_total = $query->count();
        if ($limit != '-1') {
            $query->limit($limit)->offset($start);
        }
        $users = $query->latest('id')->get()->toArray();
        $final_data = [];
        $i = 0;
        foreach ($users as $user) {
            $index = (++$i) + $start;
            $row['sl'] = $index;
            $row['id'] = $user['id'];
            $row['employee_name'] = $user['profile']['full_name'] ?? '-';
            $row['employee_type'] = $user['role']['branch_type'] ?? '-';
            $row['bank_branch_name'] = $user['role']['bank_branch']['branch_name'] ?? '-';
            $row['active_status'] = $user['active_status'] == STATUS_ON ? STATUS_ACTIVE : STATUS_INACTIVE;
            $row['action'] = '';
            $final_data[] = $row;
        }
        return ['draw' => $page, 'recordsTotal' => $_total, 'recordsFiltered' => $_total, 'data' => $final_data];
    }
    private function prepareUserSavingData($request_data)
    {
        $data['id'] = $request_data['id'];
        $data['username'] = $request_data['username'];
        $data['password'] = DEFAULT_PASSWORD;
        $data['force_change_pass'] = STATUS_ON;
        $data['active_status'] = STATUS_ON;
        if(array_key_exists('created_by',$request_data)){
            $data['created_by'] = $request_data['created_by'];
        }
        return $data;
    }
    public function employeeRegister($request_data)
    {
        try {
            $username = $request_data['username'];
            $response_duplicate_username = $this->checkDuplicateUsername($username);
            if ($response_duplicate_username['status'] == RESPONSE_STATUS_ERROR) {
                return $response_duplicate_username;
            }
            $user_data = $this->prepareUserSavingData($request_data);
            DB::beginTransaction();
            $response_user = $this->modifiedSave($user_data, []);
            if ($response_user['status'] == RESPONSE_STATUS_ERROR) {
                return $response_user;
            }
            $request_data['user_id'] = $response_user['id'];
            $modal_user_profile = new UserProfile();
            $response_user_profile = $modal_user_profile->prepareSaveProfile($request_data);
            if ($response_user_profile['status'] == RESPONSE_STATUS_ERROR) {
                DB::rollBack();
                return $response_user;
            }
            $request_data['user_profile_id'] = $response_user_profile['id'];
            $modal_user_entity_role = new UserEntityRole();
            $response_user_entity_role = $modal_user_entity_role->prepareSaveUserEntityRole($request_data);
            if ($response_user_entity_role['status'] == RESPONSE_STATUS_ERROR) {
                DB::rollBack();
                return $response_user_entity_role;
            }
            if ($response_user['status'] == RESPONSE_STATUS_SUCCESS && $response_user_profile['status'] == RESPONSE_STATUS_SUCCESS && $response_user_entity_role['status'] == RESPONSE_STATUS_SUCCESS) {
                DB::commit();
                return ['status' => RESPONSE_STATUS_SUCCESS, 'message' => __('Employee successfully register.'), 'code' => 200];
            }
            DB::rollBack();
        } catch (\Exception $e) {
            DB::rollBack();
            return ['status' => RESPONSE_STATUS_ERROR, 'message' => $e->getMessage()];
        }
    }
    public function updateUser($user_id, $request_data)
    {
        try {
            // Check if the username needs to be updated and is unique
            if (isset($request_data['username'])) {
                $username = $request_data['username'];
                $response_duplicate_username = $this->checkDuplicateUsername($username, $user_id);
                if ($response_duplicate_username['status'] == RESPONSE_STATUS_ERROR) {
                    return $response_duplicate_username;
                }
            }

            // Prepare user data for update
            $user_data = $this->prepareUserSavingData($request_data);
            DB::beginTransaction();

            // Update main user data
            $response_user = $this->modifiedSave($user_data, []);
            if ($response_user['status'] == RESPONSE_STATUS_ERROR) {
                return $response_user;
            }

            // Update or create user profile
            $modal_user_profile = UserProfile::firstOrNew(['user_id' => $user_id]);
            $response_user_profile = $modal_user_profile->prepareSaveProfile($request_data);

            if ($response_user_profile['status'] == RESPONSE_STATUS_ERROR) {
                DB::rollBack();
                return $response_user_profile;
            }
            // Update or create user entity role
            $modal_user_entity_role = UserEntityRole::firstOrNew(['user_id' => $user_id]);
            $response_user_entity_role = $modal_user_entity_role->prepareSaveUserEntityRole($request_data);
            if ($response_user_entity_role['status'] == RESPONSE_STATUS_ERROR) {
                DB::rollBack();
                return $response_user_entity_role;
            }

            // Commit the transaction if all updates succeed
            DB::commit();
            return ['status' => RESPONSE_STATUS_SUCCESS, 'message' => __('Employee successfully updated.'), 'code' => 200];

        } catch (\Exception $e) {
            dd($e);
            DB::rollBack();
            return ['status' => RESPONSE_STATUS_ERROR, 'message' => $e->getMessage()];
        }
    }

    public function getUserSetup($id)
    {
        return self::find($id);
    }


    public function checkDuplicateUsername($username, $current_user_id = null)
    {
        // Query to check for the existence of the username, excluding the current user
        $query = $this->where('username', $username);

        // If a user ID is provided, exclude that user from the check
        if ($current_user_id) {
            $query->where('id', '!=', $current_user_id);
        }

        $exists = $query->exists();

        if ($exists) {
            return ['status' => RESPONSE_STATUS_ERROR, 'message' => __('The username already exists.'), 'code' => 401];
        } else {
            return ['status' => RESPONSE_STATUS_SUCCESS, 'message' => __('The username is available.'), 'code' => 200];
        }
    }

}
