<?php

namespace App\Models;

use Carbon\Carbon;
use DateTime;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Prunable;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\DB;

class UserProfile extends AppModel
{
    use HasFactory, Prunable, SoftDeletes;
    protected $connection = 'mysql';
    protected $with = [];
    public $_profile_id = 0;
    protected $appends = [
        'profile_image',
        'full_address',
        'full_name'
    ];
    protected $validation_rules = [
        'first_name' => 'required',
        'last_name' => 'required',
        'dob' => 'required',
        'email' => 'required',
    ];
    /**
     * The model's default values for attributes.
     * @var array
     */
    protected $attributes = ['id' => 0, 'active_status' => 1];
    /**
     * The attributes that aren't mass assignable.
     * @var array
     */
    protected $guarded = ['type'];
    /**
     * Get the prunable model query.
     */
    public function prunable(): Builder
    {
        return static::where('id', '=', -100000000);
    }
    /**
     * Date parameter setup.
     */
    protected $dateFormat = 'Y-m-d H:i:s';
    protected $casts = [
        'dob'  => 'date:d-M-Y',
        'joining_date' => 'date:d-M-Y',
        'address' => 'array', // convert array to json and json to array while save and retrive
    ];
    public function setDobAttribute($value)
    {
        $this->attributes['dob'] = date('Y-m-d', strtotime($value));
    }
    public function setJoiningDateAttribute($value)
    {
        $this->attributes['joining_date'] = date('Y-m-d', strtotime($value));
    }
    /**
     * Rule parameter setup.
     */
    public function rules()
    {
        return [
            'first_name' => 'required',
            'last_name' => 'required',
            'dob' => 'required',
            'nid' => 'required|string|unique:user_profiles, nid' . $this->id,
            'driving_license' => 'nullable|string|unique:user_profiles, driving_license' . $this->id,
            'passport' => 'nullable|string|unique:user_profiles, passport' . $this->id,
            'mobile' => 'required|mobile|unique:user_profiles, mobile' . $this->id,
            'email' => 'required',
        ];
    }
    /**
     * Relation Setup
     */
    public function user()
    {
        return $this->belongsTo(User::class);
    }
    public function profile_attachment()
    {
        return $this->hasOne(FileAttachment::class, 'table_row_id')->where('x_attachment_type_id', USER_PROFILE_ATTACHMENT_TYPE_KEY);
    }
    public function signature_attachment()
    {
        return $this->hasOne(FileAttachment::class, 'table_row_id')->where('x_attachment_type_id', USER_SIGNATURE_ATTACHMENT_TYPE_KEY);
    }
    public static function boot()
    {
        parent::boot();
        static::deleting(function ($profile) {
            $profile->user()->role()->delete();
            $profile->user()->delete();
            $profile->profile_attachment()->delete();
            $profile->signature_attachment()->delete();
        });
    }
    public function getProfileImageAttribute()
    {
        $image = asset('assets/media/avatars/blank.png');
        if ($this->profile_attachment) {
            $image = base64_decode($this->profile_attachment->file_dir);
            $image =  asset('storage/' . $image);
        }
        return $image;
    }
    public function getUserSignatureAttribute()
    {
        $image = asset('assets/media/avatars/demo-signature.png');
        if ($this->signature_attachment) {
            $image = base64_decode($this->signature_attachment->file_dir);
            $image =  asset('storage/' . $image);
        }
        return $image;
    }
    public function getFullAddressAttribute()
    {
        if (empty($this->address)) {
            $this->address = [];
        }
        $_add = array_merge($this->address, [$this->address_area, $this->address_post_code]);
        return implode(', ', array_filter($_add));
    }
    public function getFullNameAttribute()
    {
        $_value = [$this->name_title, $this->first_name, $this->last_name];
        return implode(' ', array_filter($_value));
    }
    /**** Virtual Field: Ends */

    /**** Private Functions: Starts */
    function prepareSearchQuery($query, $request)
    {
        $search_keys = ['id', 'user_id'];
        foreach ($search_keys as $key) {
            if (isset($request[$key])) {
                $query->whereIn($key, is_array($request[$key]) ? $request[$key] : [$request[$key]]);
            }
        }
        if (isset($request['search_string']) && strlen($request['search_string']) > 0) {
            $query = $query->where('first_name', 'like', '%' . $request['search_string'])
                ->orwhere('last_name', 'like', "%" . $request['search_string']);
        }
        return $query;
    }
    private function prepareFullProfileData($row)
    {
        $profile = [];
        $profile['id'] = $row->id;
        $profile['active_status'] = $row->active_status ? $row->active_status : 0;
        $profile['other_role'] = !empty($row->user->other_role) ? 1 : 0;
        $profile['user_id'] = $row->user_id;
        $profile['full_name'] = $row->full_name;
        $profile['full_address'] = $row->full_address;
        $profile['designation'] = $row->designation ? $row->designation : '';
        $profile['gender'] = $row->gender ? ucfirst($row->gender) : '';
        $profile['nid'] = $row->nid ? $row->nid : '';
        $profile['passport'] = $row->passport ? $row->passport : '';
        $profile['driving_license'] = $row->driving_license ? $row->driving_license : '';
        $profile['nationality'] = $row->nationality ? $row->nationality : '';
        $profile['email'] = $row->email ? $row->email : '';
        $profile['mobile'] = $row->mobile ? $row->mobile : '';
        $profile['dob'] = $row->dob ? $row->dob : '';
        $profile['employee_code'] = $row->employee_code ? $row->employee_code : '';
        $profile['name_title'] = $row->name_title ? $row->name_title : '';
        $profile['first_name'] = $row->first_name ? $row->first_name : '';
        $profile['last_name'] = $row->last_name ? $row->last_name : '';
        $profile['address'] = $row->address ? $row->address : '';
        $profile['address_post_code'] = $row->post_code ? $row->post_code : '';
        $profile['address_area'] = $row->area ? $row->area : '';
        $profile['joining_date'] = isset($row->joining_date) ? Carbon::parse($row->joining_date)->format('d-m-Y') : null;
        $profile['photo'] = $row->profile_image;
        $profile['signature'] = '';
        $image = asset('assets/media/avatars/demo-signature.png');
        if ($row->signature_attachment) {
            $image = base64_decode($row->signature_attachment->file_dir);
            $profile['signature'] =  asset('storage/' . $image);
        }
        $profile['username'] = isset($row->user) && !empty($row->user) ? $row->user->username : '';
        return $profile;
    }
    private function prepareShortPofileData($row)
    {
        $profile = [];
        $profile['id'] = $row->id;
        $profile['user_id'] = $row->user_id;
        $profile['full_name'] = $row->full_name;
        $profile['designation'] = $row->designation ? $row->designation : '';
        $profile['email'] = $row->email ? $row->email : '';
        $profile['mobile'] = $row->mobile ? $row->mobile : '';
        $profile['employee_code'] = $row->employee_code ? $row->employee_code : '';
        $profile['photo'] = isset($row->profile_attachment) && !empty($row->profile_attachment) ? base64_decode($row->profile_attachment->file_dir) : '';
        $profile['profile_photo'] = $row->profile_image ?? '';
        $profile['file_name'] = isset($row->profile_attachment) && !empty($row->profile_attachment) ? $row->profile_attachment->file_name : '';
        return $profile;
    }
    private function getMembers($type = ENTITYTYPEID_ROLE_MANAGER, $limit, $start, $search, $entity_profile_id)
    {
        $search = $search['value'];
        $m_uer = new UserEntityRole();
        $_profiles = $m_uer->where(['entity_type_role_id' => $type, 'entity_profile_id' => $entity_profile_id])->limit($limit)->offset($start)->pluck('user_profile_id', 'user_profile_id')->toArray();
        $query = $this::with('profile_attachment')->whereIn('id', $_profiles);
        if (!empty($search)) {
            $query->where(function ($query) use ($search) {
                $query->where('first_name', 'like', "%" . $search . "%")
                    ->orwhere('last_name', 'like', "%" . $search . "%")
                    ->orWhere('designation', 'like', "%" . $search . "%")
                    ->orWhere('email', 'like', "%" . $search . "%")
                    ->orWhere('mobile', 'like', "%" . $search . "%")
                    ->orWhere('gender', 'like', "%" . $search . "%");
            });
        }
        $_total = $query->count();
        $data = $query->limit($limit)->offset($start)->get();
        return ['total' => $_total, 'data' => $data];
    }
    private function prepareDataTableDataForMember($index, $profile)
    {
        return [
            'sl' => $index,
            'id' => $profile['id'],
            'user_id' => $profile['user_id'],
            'photo' => '<td valign="middle" class="text-center">
                                <div class="d-flex align-items-center">
                                    <div class="position-relative">
                                        <div class="symbol symbol-35px symbol-circle">
                                            <img src="' . $profile['photo'] . '" alt="Profile Image" class="symbol-label">
                                        </div>
                                        <div class="' . ($profile['active_status'] == 1 ? 'bg-success' : 'bg-danger') . ' position-absolute border border-4 border-body h-15px w-15px rounded-circle translate-middle start-100 top-100 ms-n1 mt-n1"></div>
                                    </div>
                                </div>
                            </td>',
            'employee_code' => $profile['employee_code'],
            'full_name' => $profile['full_name'],
            'designation' => $profile['designation'] ?? '',
            'gender' => ucfirst($profile['gender']) ?? '',
            'email' => $profile['email'] ?? '',
            'mobile' => $profile['mobile'] ?? '',
            'saction' => ''
        ];
    }
    /**** Private Functions: Ends */

    /**** Public Functions: Starts */
    public function getUserProfile($id)
    {
        return $this->find($id);
    }
    public function saveOrUpdate($data)
    {
        return $this->modifiedSave($data, []);
    }
    public function getList($page, $limit, $start, $search)
    {
        $result = $this::all();
        $final_data = [];
        return ['draw' => $page, 'recordsTotal' => $result['total'], 'recordsFiltered' => $result['total'], 'data' => $final_data];
    }
    public function getOptions($options = [])
    {
        $_data_options = $this->all()->pluck('full_name', 'id');
        return $_data_options;
    }
    //Formatting Ends

    public function getProfile($pro_id)
    {
        return $this->with(['profile_attachment', 'signature_attachment'])->where('id', $pro_id)->first();
    }
    public function getProfileByUser($id)
    {
        return $this->with(['profile_attachment', 'signature_attachment'])->where('user_id', $id)->first();
    }
    public function getProfiles($profile_ids)
    {
        return $this->with(['profile_attachment'])->whereIn('id', $profile_ids)->get()
            ->mapWithKeys(function ($profile) {
                return [$profile->id => $profile];
            })->toArray();
    }
    public function getProfileOptions($profile_ids)
    {
        return $this->whereIn('id', $profile_ids)->get()
            ->mapWithKeys(function ($profile) {
                return [$profile->id => "[" . $profile->employee_code . "] " . $profile->full_name . " [" . $profile->mobile . "]"];
            })->toArray();
    }
    public function getMemberShortProfile($pro_id, $with_signature = false)
    {
        $member = $this->getProfile($pro_id);
        if (empty($member)) {
            return [];
        }
        $data = ['id' => $member['id'], 'user_id' => $member['user_id'], 'full_name' => $member['full_name'], 'designation' => $member['designation'], 'email' => $member['email'], 'mobile' => $member['mobile'], 'profile_photo' => $member['profile_image']];
        if ($with_signature) {
            $data['signature'] = $member['user_signature'];
        }
        return $data;
    }
    public function getManagerOptions()
    {
        $m_uer = new UserEntityRole();
        $manager_profile_ids = $m_uer->getManagerProfileIds();
        if (count($manager_profile_ids) == 0) {
            return [];
        }
        return $this->getProfileOptions($manager_profile_ids);
    }
    public function getColleagueOptions($manager_id)
    {
        $m_uer = new UserEntityRole();
        $colleague_profile_ids = $m_uer->getColleagueProfileIds($manager_id)->pluck('user_profile_id', 'user_profile_id')->toArray();
        if (count($colleague_profile_ids) == 0) {
            return [];
        }
        return $this->getProfileOptions($colleague_profile_ids);
    }
    /**
     * $manager_id : (-1 = all, 0 = no supervisor, > 0 = specific supervisor, -2 = mapped colleagues)
     */
    public function getColleagueListByManager($manager_id)
    {
        $m_uer = new UserEntityRole();
        $colleague_profile_ids = $m_uer->getColleagueProfileIds($manager_id)->pluck('id', 'user_profile_id')->toArray();
        $query = $this->with(['profile_attachment'])->whereIn('id', array_keys($colleague_profile_ids))->get();
        $final_data = [];
        foreach ($query as $row) {
            $profile = $this->prepareShortPofileData($row);
            $profile['approval_waiting'] = 0;
            $profile['map_id'] = $colleague_profile_ids[$row['id']];
            $final_data[] = $profile;
        }
        return $final_data;
    }
    public function getColleagueCountBySupervisor($manager_id)
    {
        $m_uer = new UserEntityRole();
        return count($m_uer->getColleagueProfileIds($manager_id));
    }
    public function getTeam($page, $limit, $start, $search, $auth_user)
    {
        $final_data = [[
            'sl' => 1,
            'id' => 1,
            'user_id' => 1,
            'employee_code' => 'M-01',
            'full_name' => 'Manager',
            'mobile' => '01XXXXXXXXX',
            'email' => 'example@email.com',
            'is_manager' => true,
            'photo' => '',
            'action' => '',
        ], [
            'sl' => 1.1,
            'id' => 2,
            'user_id' => 2,
            'employee_code' => 'C-01',
            'full_name' => 'Colleague',
            'mobile' => '01XXXXXXXXX',
            'email' => 'example@email.com',
            'is_manager' => false,
            'photo' => '',
            'action' => '',
        ]];
        return ['draw' => $page, 'recordsTotal' => count($final_data), 'recordsFiltered' => count($final_data), 'data' => $final_data];
    }
    public function getColleageSupervisor($colleague_profile_id)
    {
        $m_uer = new UserEntityRole();
        $manager_profile = $m_uer->getSupervisorId($colleague_profile_id);
        if (!empty($manager_profile)) {
            return $this->getMemberShortProfile($manager_profile->supervisor_user_profile_id);
        }
        return [];
    }
    public function getTeamProfile($profile_id)
    {
        $member = $this->getProfile($profile_id);
        if (empty($member) || empty($member->user)) {
            return [];
        }
        $user_id = $member->user_id;
        $role = $member->user->role->toArray();
        $member->role = $role;
        $user_role = ROLE_MANAGER;
        if ($role['entity_type_role_id'] == ENTITYTYPEID_ROLE_COLLEAGUE) {
            $user_role = ROLE_COLLEAGUE;
            $member->supervisor = $this->getMemberShortProfile($role['supervisor_user_profile_id']);
        } else if ($role['is_admin'] == 1) {
            $user_role = ROLE_ADMIN;
        } else if ($role['is_restricted_manager'] == 1) {
            $user_role = $user_role . " (" . __("Limited Access") . ")";
        }
        $member = $member->toArray();
        $member['role_display'] = ucfirst($user_role);
        $member['signature'] = '';
        $image = asset('assets/media/avatars/demo-signature.png');
        if ($member['signature_attachment']) {
            $image = base64_decode($member['signature_attachment']['file_dir']);
            $member['signature'] =  asset('storage/' . $image);
        }
        unset($member['user']['user_roles']);
        return $member;
    }
    /******* ACTION: Starts */
    public function saveUserProfile($data, $onlyProfile = false)
    {
        try {
            if (empty($data['nid'])) {
                $data['nid'] = NULL;
            }
            if (empty($data['driving_license'])) {
                $data['driving_license'] = NULL;
            }
            if (empty($data['passport'])) {
                $data['passport'] = NULL;
            }
            $response = $this->modifiedSave($data, []);
            if ($response['status'] != 'success') {
                return $response;
            }
            if ($onlyProfile) {
                return $response;
            }
            $user_profile_id = $response['id'];
            $file_attachment = new FileAttachment();
            $send_data['table_name'] = USER_PROFILE_TABLE_NAME;
            $send_data['table_row_id'] = $user_profile_id;
            $user_entity_role = new UserEntityRole();
            $data['user_profile_id'] = $user_profile_id;
            $data['entity_profile_id'] = 1;
            $data['is_default_role'] = 1;
            $response = $user_entity_role->saveUserEntityRole($data);

            if (!empty($data['profile_image'])) {
                $send_data['image'] = $data['profile_image'];
                $send_data['original_name'] = isset($data['profile_image_name']) ? $data['profile_image_name'] : '';
                $send_data['x_attachment_type_id'] = USER_PROFILE_ATTACHMENT_TYPE_KEY;
                $f_response = $file_attachment->base64Upload($send_data);
            }
            if (!empty($data['signature_image'])) {
                $send_data['image'] = $data['signature_image'];
                $send_data['original_name'] = isset($data['signature_image_name']) ? $data['signature_image_name'] : '';
                $send_data['x_attachment_type_id'] = USER_SIGNATURE_ATTACHMENT_TYPE_KEY;
                $f_response = $file_attachment->base64Upload($send_data);
            }
            if (isset($response['status']) && $response['status'] == 'success') {
                $response = [
                    'status' => 'success',
                    'message' => 'Saved successfully.',
                    'code' => 200
                ];
            }
        } catch (\Exception $e) {
            $response = [
                'success' => 'error',
                'message' => 'An error occurred while saving user profile.',
                'error' => $e->getMessage(),
                'code' => 500
            ];
        }
        return $response;
    }

    public function deleteUserProfile($id)
    {
        return $this->where('id', $id)->delete();
    }
    /******* ACTION: Ends */

    public function getUserEmail($rd)
    {
        try {
            $search_string = $rd['search_string'];
            $search_explode = array_diff(explode(' ', $search_string), ['mr', 'miss', 'mrs', 'ms']);
            foreach ($search_explode as $search_text) {
                $data = $this->searchEmailByFirstNameOrLastName($search_text);
                if ($data) {
                    return ['status' => RESPONSE_STATUS_SUCCESS, 'message' => __('Email successfully found.'), 'data' => $data];
                }
            }
            return ['status' => RESPONSE_STATUS_ERROR, 'message' => __('Email not found.'), 'data' => ''];
        } catch (\Exception $e) {
            return ['status' => RESPONSE_STATUS_ERROR, 'message' => $e->getMessage()];
        }
    }

    private function searchEmailByFirstNameOrLastName($search_text)
    {
        $email = $this->select(['id', 'email'])
            ->where(function ($query) use ($search_text) {
                $query->where('first_name', 'like', '%' . $search_text . '%')
                    ->orWhere('last_name', 'like', '%' . $search_text . '%');
            })->first();
        return $email ? ['id' => $email->id, 'email' => $email->email] : null;
    }
    public function userProfileStatusUpdate($user_id, $active_status)
    {
        try {
            $query = $this->select('*');
            $user_profile = $this->prepareSearchQuery($query, ['user_id' => $user_id])->first();
            if ($user_profile->update(['active_status' => $active_status])) {
                return ['status' => RESPONSE_STATUS_SUCCESS, 'message' => __('Successfullly profile actived.')];
            }
        } catch (\Exception $e) {
            return ['status' => RESPONSE_STATUS_ERROR, 'message' => $e->getMessage()];
        }
    }
    ######################
    ##### user setup #####
    ######################
    public function prepareSaveProfile($request_data)
    {
        try {
            $exists_profile = $this->select('id', 'user_id')->where('user_id', $request_data['user_id'])->first();
            $profile_data['id'] = $exists_profile->id ?? 0;
            $profile_data['user_id'] = $request_data['user_id'];
            $profile_data['name_title'] = $request_data['name_title'];
            $profile_data['first_name'] = $request_data['first_name'];
            $profile_data['last_name'] = $request_data['last_name'];
            $profile_data['gender'] = $request_data['gender'];
            $profile_data['designation'] = $request_data['designation'];
            $profile_data['mobile'] = $request_data['mobile'];
            $profile_data['email'] = $request_data['email'];
            if(array_key_exists('created_by',$request_data)){
                $profile_data['created_by'] = $request_data['created_by'];
            }
            DB::beginTransaction();
            $response_user = $this->modifiedSave($profile_data, []);
            if ($response_user['status'] == RESPONSE_STATUS_ERROR) {
                DB::rollBack();
                return $response_user;
            }
            DB::commit();
            return $response_user;
        } catch (\Exception $e) {
            return $this->responseWithError($e->getMessage())->original;
        }
    }
}
