<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Prunable;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Validation\Rule;

class XDocument extends AppModel
{
    use HasFactory, Prunable, SoftDeletes;
    protected $connection = 'mysql';
    protected $with = [];
    protected $attributes = ['id' => 0];
    protected $guarded = ['type'];
    protected $dateFormat = 'Y-m-d H:i:s';
    protected $dates = [];
    public function rules($id = null)
    {
        return [
            'document_title' => [
                'required',
                'string',
                'max:255',
                Rule::unique('x_service_documents', 'document_title')->ignore($id),
            ],
            'document_description' => 'required',
            'document_group' => 'nullable',
            'verify_role_ids' => 'nullable',
        ];
    }

    protected $casts = [
        'created_at' => 'date:d-M-Y',
        'updated_at' => 'date:d-M-Y',
        'verify_role_ids' => 'array'
    ];

    public function prunable(): Builder
    {
        return static::where('id', '=', -100000000);
    }
    private function documentDeletionRoles($query)
    {
        return true;
    }

    public function files()
    {
        return $this->hasMany(ClientServiceDocument::class, 'x_document_id');
    }
    public function af_documents()
    {
        return $this->hasMany(FileAttachment::class, 'table_row_id');
    }

    public function map_service_document()
    {
        return $this->hasOne(MapServiceDocument::class, 'x_document_id', 'id');
    }

    private function prepareSearchQuery($query, $request)
    {
        $search_keys = ['id', 'document_group'];
        foreach ($search_keys as $key) {
            if (isset($request[$key]) && !empty($request[$key])) {
                $query->whereIn($key, is_array(($request[$key])) ? $request[$key] : [$request[$key]]);
            }
        }
        if (isset($request['search_string']) && strlen($request['search_string']) > 0) {
            $query->where('document_title', 'like', "%" . $request['search_string'] . "%");
        }
        return $query;
    }

    public function getDocuments()
    {
        return $this->all();
    }

    public function getDocumentTitles()
    {
        $documents = [
            'identity_documents' => [],
            'contract_documents' => [],
        ];
        $all_documents = $this->select("id", "document_title", "document_group", "email_template_id")
            ->whereNotNull('document_group')
            ->where('document_group', '!=', '')
            ->get();
        foreach ($all_documents as $document) {
            if ($document->document_group == IDENTITY_DOCUMENTS) {
                $documents['identity_documents'][$document['id']] = $document->document_title;
            } elseif ($document->document_group == CONTRACT_DOCUMENTS) {
                $documents['contract_documents'][] = [
                    "id" => $document->id,
                    "email_template_id" => $document->email_template_id,
                    "document_title" => $document->document_title ? $document->document_title : ''
                ];
            }
        }
        return $documents;
    }

    public function checkRoleAutoVerifyOrNot($role_id, $id)
    {
        $document = $this->getDocument($id);
        if ($document) {
            if (!empty($document->verify_role_ids)) {
                $document = $document->verify_role_ids;
                return in_array($role_id, $document);
            }
        }
        return false;
    }

    public function getOwnerFiles($owner_id)
    {
        return ClientServiceDocument::with('file_attachment')->where('x_document_id', $this->id)->where('company_owner_id', $owner_id)->get();
    }

    public function getIdentifyDocuments($client_profile_id)
    {
        return $this->with(['files' => function ($query) use ($client_profile_id) {
            $query->where('client_profile_id', $client_profile_id);
        }])->where('document_group', IDENTITY_DOCUMENTS)->get();
    }

    public function getDocument($id)
    {
        return $this->findOrFail($id);
    }

    public function getList($page, $limit, $start, $search)
    {
        $query = $this::select(['id', 'document_title', 'document_description', 'document_group', 'verify_role_ids']);
        $query = $this->prepareSearchQuery($query, ['search_string' => $search['value']]);
        $x_documents = $query->limit($limit)->offset($start)->latest('id')->get()->toArray();
        $_total = $this->count();
        $final_data = [];
        $i = 0;
        $roles = new EntityTypeRole();
        foreach ($x_documents as $x_document) {
            $index = (++$i) + $start;
            $row = [
                'sl' => $index,
                'id' => $x_document['id'],
                'document_title' => $x_document['document_title'],
                'document_description' => $x_document['document_description'],
                'document_group' => $x_document['document_group'],
                'roles' =>  $roles->getRoleByNames($x_document['verify_role_ids']),
                'daction' => '',
            ];
            $final_data[] = $row;
        }
        return ['draw' => $page, 'recordsTotal' => $_total, 'recordsFiltered' => $_total, 'data' => $final_data];
    }

    public function saveOrUpdate(array $data)
    {
        return $this->modifiedSave($data, []);
    }

    public function deleteDocument($id)
    {
        $query = $this->getDocument($id);
        $delete_role = $this->documentDeletionRoles($query);
        if ($delete_role === true) {
            $query->delete();
            return ['message' => __('Deleted Successfully'), 'status' => 'success'];
        } else {
            return ['message' => __($delete_role), 'status' => 'error'];
        }
    }

    public function getOptions($request = null)
    {
        try {
            $query = $this::select('id', 'document_title');
            return $this->prepareSearchQuery($query, ['document_group' => $request['document_group'] ?? '', 'search_string' => $request['value'] ?? ''])->pluck('document_title', 'id');
        } catch (\Exception $e) {
            return [];
        }
    }

    public function getDocumentByIds($ids)
    {
        $query = $this->select('id', 'document_title', 'document_group');
        $query = $this->prepareSearchQuery($query, [
            'id' => $ids,
        ])->get()->toArray();
        return $query;
    }

    public function getAFDocuments()
    {
        $query = $this->select(['*']);
        return $this->prepareSearchQuery($query, ['document_group' => ACCOUNTENCY_FIRM_GORUP_NAME])->get()->toArray();
    }

    public function getXDocuments($condtions, $select = ['*'])
    {
        $query = $this->select($select);
        return $this->prepareSearchQuery($query, $condtions);
    }
}
