<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Prunable;
use Illuminate\Database\Eloquent\SoftDeletes;

class XFolder extends AppModel
{
    use HasFactory, Prunable, SoftDeletes;
    protected $connection = 'mysql';
    protected $with = [];
    protected $attributes = ['id' => 0];
    protected $guarded = ['type'];
    protected $dateFormat = 'Y-m-d H:i:s';
    protected $dates = [];
    public static function rules()
    {
        return [
            'name' => ['required'],
        ];
    }

    protected $casts = [
        'created_at' => 'date:d-m-Y',
        'updated_at' => 'date:d-m-Y',
    ];

    protected $appends = ['total_file'];
    public function getTotalFileAttribute()
    {
        return $this->getAllChildFileCounts($this->id);
    }

    public function prunable(): Builder
    {
        return static::where('id', '=', -100000000);
    }

    private function prepareSearchQuery($query, $request)
    {
        $search_keys = ['id', 'parent_id', 'status'];
        foreach ($search_keys as $key) {
            if (isset($request[$key]) && intval($request[$key]) > 0) {
                $query->whereIn($key, is_array(($request[$key])) ? $request[$key] : [$request[$key]]);
            }
        }
        if (isset($request['search_string']) && strlen($request['search_string']) > 0) {
            $query->where('name', 'like', "%" . $request['search_string'] . "%");
        }
        return $query;
    }

    private function folderDeletionRoles($query)
    {
        $attachment = new FileAttachment();
        $conditions = [
            ['table_name', '=', "x_folders"],
            ['table_row_id', '=', $query->id],
        ];
        $attachment = $attachment->getConditionWiseAttachments($conditions)->count();
        if ($attachment > 0) {
            return false;
        }
        return true;
    }

    private function getAllParents($folder, &$parents)
    {
        if ($folder->parent_id !== null) {
            $parent = $this->getFolder($folder->parent_id);
            if ($parent) {
                $parents[] = $parent;
                $this->getAllParents($parent, $parents);
            }
        }
        return $parents;
    }

    private function getAllParentNames($folder, &$parentNames)
    {
        if ($folder->parent_id !== null) {
            $parent = $this->getFolder($folder->parent_id);
            if ($parent) {
                $this->getAllParentNames($parent, $parentNames);
                $parentNames[] = $parent->name;
            }
        }
        return $parentNames;
    }

    private function forceDeleteDirectory($directoryPath)
    {
        if (is_dir($directoryPath)) {
            $files = scandir($directoryPath);
            foreach ($files as $file) {
                if ($file !== '.' && $file !== '..') {
                    $filePath = $directoryPath . '/' . $file;
                    if (is_dir($filePath)) {
                        $this->forceDeleteDirectory($filePath);
                    } else {
                        unlink($filePath);
                    }
                }
            }
            rmdir($directoryPath);
        }
    }

    public function subfolders()
    {
        return $this->hasMany(XFolder::class, 'parent_id');
    }
    public function parent()
    {
        return $this->belongsTo(XFolder::class, 'parent_id');
    }
    public function fileAttachments()
    {
        return $this->belongsToMany(FileAttachment::class, 'file_attachment_x_folder');
    }

    public function getChildFolders()
    {
        return $this->hasMany(XFolder::class, 'parent_id', 'id');
    }

    public function getFolder($id)
    {
        return $this->findOrFail($id);
    }

    public function saveOrUpdate(array $data)
    {
        return $this->modifiedSave($data, []);
    }

    public function deleteFolder($id)
    {
        $folder = $this->getFolder($id);
        $delete_role = $this->folderDeletionRoles($folder);
        if ($delete_role === true) {
            $parents = [];
            $this->getAllParentNames($folder, $parents);
            $path = implode('/', $parents) . '/' . $folder->name;
            $storagePath = storage_path(IMAGE_PREFIX . IMAGE_PATH . '/' . APP_MODE . '/' . FOLDER_PATH . '/' . $path);

            if (is_dir($storagePath)) {
                $files = scandir($storagePath);
                $fileCount = count($files);
                // Check if the folder is empty (only contains "." and "..")
                if ($fileCount === 2) {
                    if (rmdir($storagePath)) {
                        $folder->delete();
                    }
                } else {
                    $this->forceDeleteDirectory($storagePath);
                    $folder->delete();
                }
            } else {
                $folder->delete();
            }

            return ['message' => __('Deleted Successfully'), 'status' => 'success'];
        } else {
            return ['message' => __($delete_role), 'status' => 'error'];
        }
    }

    public function saveFolderDataPrepare(array $data)
    {
        $data['parent_id'] = $data['parent_id'] == 0 ? null : $data['parent_id'];
        $folder = $this->saveOrUpdate($data);
        $path = $data['name'];
        if ($folder['status'] == "success") {
            $parents = [];
            $this->getAllParentNames($folder['data'], $parents);
            $path = implode('/', $parents) . '/' . $data['name'];
        }
        $storagePath = storage_path(IMAGE_PREFIX . IMAGE_PATH . '/' . APP_MODE . '/' . FOLDER_PATH . '/' . $path);
        mkdir($storagePath, 0755, true);
        return $folder;
    }

    public function getFolderByChild($id)
    {
        $folder['child'] = $this->getFolder($id);
        $folder['parents'] = [];
        if ($folder) {
            $folder['parents'] = $this->getAllParents($folder['child'], $folder['parents']);
        }
        return $folder;
    }

    public function getFolders($search)
    {
        $search['value'] = isset($search['value']) ? $search['value'] : '';
        $query = $this->with('subfolders')->whereNull('parent_id');
        $folders = $this->prepareSearchQuery($query, ['search_string' => $search['value']])->get();
        return $folders;
    }

    public function getFolderWithParentsName($id)
    {
        $folder['child'] = $this->getFolder($id);
        $folder['parents'] = '';
        if ($folder) {
            $parents = [];
            $this->getAllParentNames($folder['child'], $parents);
            $folder['parents'] = implode('/', $parents) . '/';
        }
        $folder['parents'] = $folder['parents'] . $folder['child']->name . '/';
        return $folder;
    }

    public function getAllChildFileCounts($id)
    {
        $attachment = new FileAttachment();
        $folder = $this->getFolder($id);
        $conditions = [
            ['table_name', '=', "x_folders"],
            ['table_row_id', '=', $id],
        ];
        $total = $attachment->getConditionWiseAttachments($conditions)->count();
        if ($folder) {
            $childFolders = $folder->getChildFolders;
            foreach ($childFolders as $childFolder) {
                $total += $this->getAllChildFileCounts($childFolder->id);
            }
        }
        return $total;
    }
}
